# -*- coding: utf-8 -*-


from wechatpy import messages
from wechatpy.fields import IntegerField, StringField

MESSAGE_TYPES = {}


def register_message(msg_type):
    def register(cls):
        MESSAGE_TYPES[msg_type] = cls
        return cls

    return register


@register_message("text")
class TextMessage(messages.TextMessage):
    agent = IntegerField("AgentID", 0)


@register_message("image")
class ImageMessage(messages.ImageMessage):
    agent = IntegerField("AgentID", 0)


@register_message("voice")
class VoiceMessage(messages.VoiceMessage):
    agent = IntegerField("AgentID", 0)


@register_message("shortvideo")
class ShortVideoMessage(messages.ShortVideoMessage):
    agent = IntegerField("AgentID", 0)


@register_message("video")
class VideoMessage(messages.VideoMessage):
    agent = IntegerField("AgentID", 0)


@register_message("location")
class LocationMessage(messages.LocationMessage):
    agent = IntegerField("AgentID", 0)


@register_message("link")
class LinkMessage(messages.LinkMessage):
    agent = IntegerField("AgentID", 0)
    pic_url = StringField("PicUrl")



# -*- coding: utf-8 -*-
from operator import itemgetter

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatInvoice(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/card/invoice/"

    def get_url(self):
        """
        获取自身的开票平台识别码

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Invoicing_Platform_API_List.html

        :return:该开票平台专用的授权链接
        """
        return self._post(
            "seturl",
            data={},
            result_processor=itemgetter("invoice_url"),
        )

    def create_card(self, base_info, payee, invoice_type, detail=None):
        """
        创建发票卡券模板
        注意这里的对象和会员卡有类似之处，但是含义有不同。创建发票卡券模板是创建发票卡券的基础。

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Invoicing_Platform_API_List.html

        :param base_info:发票卡券模板基础信息
        :type base_info: dict
        :param payee: 收款方（开票方）全称，显示在发票详情内。建议一个收款方对应一个发票卡券模板
        :param invoice_type: 发票类型
        :param detail: 备注详情
        :return: 发票卡券模板的编号，用于后续该商户发票生成后，作为必填参数在调用插卡接口时传入
        """
        # FIXME 新版接口已经移除了 detail 参数，请确认此参数是否需要保留
        return self._post(
            "platform/createcard",
            data={
                "invoice_info": {
                    "base_info": base_info,
                    "payee": payee,
                    "type": invoice_type,
                    "detail": detail,
                },
            },
            result_processor=lambda x: x["card_id"],
        )

    def get_auth_url(
        self,
        s_pappid,
        order_id,
        money,
        timestamp,
        source,
        ticket,
        auth_type,
        redirect_url=None,
    ):
        """
        获取授权页链接

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html

        :param s_pappid: 开票平台在微信的标识号，商户需要找开票平台提供
        :param order_id: 订单id，在商户内单笔开票请求的唯一识别号
        :param money: 订单金额，以分为单位
        :type money: int
        :param timestamp: Unix 时间戳
        :type timestamp: int
        :param source: 开票来源。app: App开票, web: 微信H5开票, wap: 普通网页开票, wxa：小程序开发票
        :param ticket: 根据获取授权ticket接口取得
        :param auth_type: 授权类型。0: 开票授权，1: 填写字段开票授权，2: 领票授权
        :type auth_type: int
        :param redirect_url: 授权成功后跳转页面。本字段只有在source为H5的时候需要填写。
        :return: 获取授权页链接
        """
        if source not in {"app", "web", "wap"}:
            raise ValueError('Unsupported source. Valid sources are "app", "web" or "wap"')
        if source == "web" and redirect_url is None:
            raise ValueError("redirect_url is required if source is web")
        if auth_type not in {0, 1, 2}:
            raise ValueError("Unsupported auth type. Valid auth types are 0, 1 or 2")
        return self._post(
            "getauthurl",
            data={
                "s_pappid": s_pappid,
                "order_id": order_id,
                "money": money,
                "timestamp": timestamp,
                "source": source,
                "ticket": ticket,
                "type": auth_type,
                "redirect_url": redirect_url,
            },
            result_processor=itemgetter("auth_url"),
        )

    def set_auth_field(self, user_field, biz_field):
        """
        设置授权页字段信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html

        :param user_field: 授权页个人发票字段
        :type user_field: dict
        :param biz_field: 授权页单位发票字段
        :type biz_field: dict
        """
        return self._post(
            "setbizattr",
            params={
                "action": "set_auth_field",
            },
            data={
                "auth_field": {
                    "user_field": user_field,
                    "biz_field": biz_field,
                },
            },
        )

    def get_auth_field(self):
        """
        获取授权页字段信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html

        :return: 授权页的字段设置
        :rtype: dict
        """
        return self._post(
            "setbizattr",
            params={
                "action": "get_auth_field",
            },
            data={},
        )

    def get_auth_data(self, s_pappid, order_id):
        """
        查询授权数据

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html

        :param s_pappid: 开票平台在微信的标识，由开票平台告知商户
        :param order_id: 订单id，在商户内单笔开票请求的唯一识别号
        :return: 用户的开票信息
        :rtype: dict
        """
        return self._post(
            "getauthdata",
            data={
                "s_pappid": s_pappid,
                "order_id": order_id,
            },
        )

    def reject_insert(self, s_pappid, order_id, reason, redirect_url=None):
        """
        拒绝用户的开发票请求

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html

        :param s_pappid: 开票平台在微信的标识号，商户需要找开票平台提供
        :param order_id: 订单id，在商户内单笔开票请求的唯一识别号
        :param reason: 商家解释拒绝开票的原因，如重复开票，抬头无效、已退货无法开票等
        :param redirect_url: 跳转链接，引导用户进行下一步处理，如重新发起开票、重新填写抬头、展示订单情况等
        """
        return self._post(
            "rejectinsert",
            data={
                "s_pappid": s_pappid,
                "order_id": order_id,
                "reason": reason,
                "url": redirect_url,
            },
        )

    def insert(self, order_id, card_id, appid, card_ext):
        """
        将电子发票卡券插入用户卡包

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Invoicing_Platform_API_List.html

        :param order_id: 发票order_id，既商户给用户授权开票的订单号
        :param card_id: 发票卡券模板的编号
        :param appid: 商户 AppID
        :param card_ext: 发票具体内容
        :type card_ext: dict
        :return: 随机防重字符串，以及用户 Open ID
        """
        return self._post(
            "insert",
            data={
                "order_id": order_id,
                "card_id": card_id,
                "appid": appid,
                "card_ext": card_ext,
            },
        )

    def upload_pdf(self, pdf):
        """
        上传电子发票中的消费凭证 PDF

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Invoicing_Platform_API_List.html

        :param pdf: 要上传的 PDF 文件，一个 File-object
        :return: 64位整数，在将发票卡券插入用户卡包时使用用于关联pdf和发票卡券。有效期为3天。
        """
        return self._post(
            "platform/setpdf",
            files={
                "pdf": pdf,
            },
            result_processor=itemgetter("s_media_id"),
        )

    def get_pdf(self, s_media_id):
        """
        查询已上传的 PDF

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Invoicing_Platform_API_List.html

        :param s_media_id: PDF 文件上传时的 s_media_id
        :return: PDF 文件的 URL，以及过期时间
        :rtype: dict
        """
        return self._post(
            "platform/getpdf",
            params={
                "action": "get_url",
            },
            data={
                "s_media_id": s_media_id,
            },
        )

    def update_status(self, card_id, code, reimburse_status):
        """
        更新发票卡券的状态

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Invoicing_Platform_API_List.html

        :param card_id: 发票卡券模板的编号
        :param code: 发票卡券的编号
        :param reimburse_status: 发票报销状态
        """
        return self._post(
            "platform/updatestatus",
            data={
                "card_id": card_id,
                "code": code,
                "reimburse_status": reimburse_status,
            },
        )

    def set_contact(self, phone, time_out):
        """
        商户获取授权链接之前，需要先设置商户的联系方式
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html#17

        :param phone: 联系电话
        :param time_out: 开票超时时间
        """
        return self._post(
            "setbizattr",
            params={
                "action": "set_contact",
            },
            data={
                "contact": {
                    "phone": phone,
                    "time_out": time_out,
                },
            },
        )

    def get_contact(self):
        """
        商户获取授权链接之前，需要先设置商户的联系方式
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html#17

        """
        return self._post(
            "setbizattr",
            params={
                "action": "get_contact",
            },
            data={},
        )

    def set_pay_mch(self, mchid, s_pappid):
        """
        关联商户号与开票平台，设置支付后开票

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html#17

        :param mchid: 微信支付商户号
        :param s_pappid: 开票平台在微信的标识号，商户需要找开票平台提供
        """
        return self._post(
            "setbizattr",
            params={
                "action": "set_pay_mch",
            },
            data={
                "paymch_info": {
                    "mchid": mchid,
                    "s_pappid": s_pappid,
                },
            },
        )

    def get_pay_mch(self):
        """
        查询商户号与开票平台关联情况

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Vendor_API_List.html

        :return: mchid 和 s_pappid
        :rtype: dict
        """
        return self._post(
            "setbizattr",
            params={
                "action": "get_pay_mch",
            },
            data={},
        )

    def get_reimburse(self, card_id, encrypt_code):
        """
        报销方查询发票信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/Auto-print/API_Documentation.html#_5-2-%E8%AF%B7%E6%B1%82%E6%96%B9%E5%BC%8F

        :param card_id: 发票卡券的 Card ID
        :param encrypt_code: 发票卡券的加密 Code
        :return: 电子发票的结构化信息
        :rtype: dict
        """
        return self._post(
            "reimburse/getinvoiceinfo",
            data={
                "card_id": card_id,
                "encrypt_code": encrypt_code,
            },
        )

    def update_reimburse(self, card_id, encrypt_code, reimburse_status):
        """
        报销方更新发票信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Reimburser_API_List.html

        :param card_id: 发票卡券的 Card ID
        :param encrypt_code: 发票卡券的加密 Code
        :param reimburse_status: 发票报销状态
        """
        return self._post(
            "reimburse/updateinvoicestatus",
            data={
                "card_id": card_id,
                "encrypt_code": encrypt_code,
                "reimburse_status": reimburse_status,
            },
        )

    def batch_update_reimburse(self, openid, reimburse_status, invoice_list):
        """
        报销方批量更新发票信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/E_Invoice/Reimburser_API_List.html

        :param openid: 用户的 Open ID
        :param reimburse_status: 发票报销状态
        :param invoice_list: 发票列表
        :type invoice_list: list[dict]
        """
        return self._post(
            "reimburse/updatestatusbatch",
            data={
                "openid": openid,
                "reimburse_status": reimburse_status,
                "invoice_list": invoice_list,
            },
        )

    def get_user_title_url(
        self,
        user_fill,
        title=None,
        phone=None,
        tax_no=None,
        addr=None,
        bank_type=None,
        bank_no=None,
        out_title_id=None,
    ):
        """
        获取添加发票链接
        获取链接，发送给用户。用户同意以后，发票抬头信息将会录入到用户微信中

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/Quick_issuing/Interface_Instructions.html

        :param user_fill: 企业设置抬头为0，用户自己填写抬头为1
        :type user_fill: bool
        :param title: 抬头，当 user_fill 为 False 时必填
        :param phone: 联系方式
        :param tax_no: 税号
        :param addr: 地址
        :param bank_type: 银行类型
        :param bank_no: 银行号码
        :param out_title_id: 开票码
        :return: 添加发票的链接
        """
        if user_fill and title is None:
            raise ValueError("title is required when user_fill is False")
        return self._post(
            "biz/getusertitleurl",
            data={
                "user_fill": int(user_fill),
                "title": title,
                "phone": phone,
                "tax_no": tax_no,
                "addr": addr,
                "bank_type": bank_type,
                "bank_no": bank_no,
                "out_title_id": out_title_id,
            },
            result_processor=itemgetter("url"),
        )

    def get_select_title_url(self, attach=None):
        """
        获取商户专属开票链接
        商户调用接口，获取链接。用户扫码，可以选择抬头发给商户。可以将链接转成二维码，立在收银台。

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/Quick_issuing/Interface_Instructions.html

        :param attach: 附加字段，用户提交发票时会发送给商户
        :return: 商户专属开票链接
        """
        return self._post(
            "biz/getselecttitleurl",
            data={
                "attach": attach,
            },
            result_processor=itemgetter("url"),
        )

    def scan_title(self, scan_text):
        """
        根据扫描码，获取用户发票抬头
        商户扫用户“我的—个人信息—我的发票抬头”里面的抬头二维码后，通过调用本接口，可以获取用户抬头信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Invoice/Quick_issuing/Interface_Instructions.html

        :param scan_text: 扫码后获取的文本
        :return: 用户的发票抬头数据
        :rtype: dict
        """
        return self._post(
            "scantitle",
            data={
                "scan_text": scan_text,
            },
        )



# -*- coding: utf-8 -*-
from wechatpy.fields import BaseField, FloatField, IntegerField, StringField
from wechatpy.messages import BaseMessage

EVENT_TYPES = {}


class BaseEvent(BaseMessage):
    """Base class for Wechat Work events"""

    type = "event"
    event = ""

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        # 注册子类
        if cls.event:
            EVENT_TYPES[cls.event] = cls


class SubscribeEvent(BaseEvent):
    """
    成员关注事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E6%88%90%E5%91%98%E5%85%B3%E6%B3%A8%E5%8F%8A%E5%8F%96%E6%B6%88%E5%85%B3%E6%B3%A8%E4%BA%8B%E4%BB%B6
    """

    agent = IntegerField("AgentID", 0)
    key = StringField("EventKey", "")
    event = "subscribe"


class UnsubscribeEvent(BaseEvent):
    """
    成员取消关注事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E6%88%90%E5%91%98%E5%85%B3%E6%B3%A8%E5%8F%8A%E5%8F%96%E6%B6%88%E5%85%B3%E6%B3%A8%E4%BA%8B%E4%BB%B6
    """

    agent = IntegerField("AgentID", 0)
    event = "unsubscribe"


class ClickEvent(BaseEvent):
    """
    点击菜单拉取消息事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E7%82%B9%E5%87%BB%E8%8F%9C%E5%8D%95%E6%8B%89%E5%8F%96%E6%B6%88%E6%81%AF%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    agent = IntegerField("AgentID", 0)
    key = StringField("EventKey")
    event = "click"


class ViewEvent(BaseEvent):
    """
    点击菜单跳转链接事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E7%82%B9%E5%87%BB%E8%8F%9C%E5%8D%95%E8%B7%B3%E8%BD%AC%E9%93%BE%E6%8E%A5%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    agent = IntegerField("AgentID", 0)
    url = StringField("EventKey")
    event = "view"


class LocationEvent(BaseEvent):
    """
    上报地理位置事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E4%B8%8A%E6%8A%A5%E5%9C%B0%E7%90%86%E4%BD%8D%E7%BD%AE
    """

    agent = IntegerField("AgentID", 0)
    latitude = FloatField("Latitude", 0.0)
    longitude = FloatField("Longitude", 0.0)
    precision = FloatField("Precision", 0.0)
    event = "location"


class BaseScanCodeEvent(BaseEvent):
    key = StringField("EventKey")
    scan_code_info = BaseField("ScanCodeInfo", {})

    @property
    def scan_type(self):
        return self.scan_code_info["ScanType"]

    @property
    def scan_result(self):
        return self.scan_code_info["ScanResult"]


class ScanCodePushEvent(BaseScanCodeEvent):
    """
    扫码推事件的事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E6%89%AB%E7%A0%81%E6%8E%A8%E4%BA%8B%E4%BB%B6%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    agent = IntegerField("AgentID", 0)
    event = "scancode_push"


class ScanCodeWaitMsgEvent(BaseScanCodeEvent):
    """
    扫码推事件且弹出“消息接收中”提示框的事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E6%89%AB%E7%A0%81%E6%8E%A8%E4%BA%8B%E4%BB%B6%E4%B8%94%E5%BC%B9%E5%87%BA%E2%80%9C%E6%B6%88%E6%81%AF%E6%8E%A5%E6%94%B6%E4%B8%AD%E2%80%9D%E6%8F%90%E7%A4%BA%E6%A1%86%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    agent = IntegerField("AgentID", 0)
    event = "scancode_waitmsg"


class BasePictureEvent(BaseEvent):
    key = StringField("EventKey")
    pictures_info = BaseField("SendPicsInfo", {})

    @property
    def count(self):
        return int(self.pictures_info["Count"])

    @property
    def pictures(self):
        if self.pictures_info["PicList"]:
            items = self.pictures_info["PicList"]["item"]
            if self.count > 1:
                return items
            return [items]
        return []


class PicSysPhotoEvent(BasePictureEvent):
    """
    弹出系统拍照发图事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E5%BC%B9%E5%87%BA%E7%B3%BB%E7%BB%9F%E6%8B%8D%E7%85%A7%E5%8F%91%E5%9B%BE%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    agent = IntegerField("AgentID", 0)
    event = "pic_sysphoto"


class PicPhotoOrAlbumEvent(BasePictureEvent):
    """
    弹出拍照或相册发图事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E5%BC%B9%E5%87%BA%E6%8B%8D%E7%85%A7%E6%88%96%E8%80%85%E7%9B%B8%E5%86%8C%E5%8F%91%E5%9B%BE%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    agent = IntegerField("AgentID", 0)
    event = "pic_photo_or_album"


class PicWeChatEvent(BasePictureEvent):
    """
    弹出微信相册发图器事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E5%BC%B9%E5%87%BA%E5%BE%AE%E4%BF%A1%E7%9B%B8%E5%86%8C%E5%8F%91%E5%9B%BE%E5%99%A8%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    agent = IntegerField("AgentID", 0)
    event = "pic_weixin"


class LocationSelectEvent(BaseEvent):
    """
    弹出地理位置选择器的事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E5%BC%B9%E5%87%BA%E5%9C%B0%E7%90%86%E4%BD%8D%E7%BD%AE%E9%80%89%E6%8B%A9%E5%99%A8%E7%9A%84%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    key = StringField("EventKey")
    location_info = BaseField("SendLocationInfo", {})
    agent = IntegerField("AgentID", 0)
    event = "location_select"

    @property
    def location_x(self):
        return self.location_info["Location_X"]

    @property
    def location_y(self):
        return self.location_info["Location_Y"]

    @property
    def location(self):
        return self.location_x, self.location_y

    @property
    def scale(self):
        return self.location_info["Scale"]

    @property
    def label(self):
        return self.location_info["Label"]

    @property
    def poiname(self):
        return self.location_info["Poiname"]


class EnterAgentEvent(BaseEvent):
    """
    用户进入应用的事件推送

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E8%BF%9B%E5%85%A5%E5%BA%94%E7%94%A8
    """

    agent = IntegerField("AgentID", 0)
    event = "enter_agent"


class BatchJobResultEvent(BaseEvent):
    """
    异步任务完成事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E5%BC%82%E6%AD%A5%E4%BB%BB%E5%8A%A1%E5%AE%8C%E6%88%90%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    event = "batch_job_result"
    batch_job = BaseField("BatchJob")

    @property
    def job_id(self):
        return self.batch_job["JobId"]

    @property
    def job_type(self):
        return self.batch_job["JobType"]

    @property
    def err_code(self):
        return self.batch_job["ErrCode"]

    @property
    def err_msg(self):
        return self.batch_job["ErrMsg"]


class OpenApprovalChangeEvent(BaseEvent):
    """
    审批状态通知事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E5%AE%A1%E6%89%B9%E7%8A%B6%E6%80%81%E9%80%9A%E7%9F%A5%E4%BA%8B%E4%BB%B6
    """

    event = "open_approval_change"
    agent = IntegerField("AgentID", 0)
    approval_info = BaseField("ApprovalInfo")

    @property
    def third_no(self):
        return self.approval_info["ThirdNo"]

    @property
    def open_sp_name(self):
        return self.approval_info["OpenSpName"]

    @property
    def open_template_id(self):
        return self.approval_info["OpenTemplateId"]

    @property
    def open_sp_status(self):
        return self.approval_info["OpenSpStatus"]

    @property
    def apply_time(self):
        return self.approval_info["ApplyTime"]

    @property
    def apply_user_name(self):
        return self.approval_info["ApplyUserName"]

    @property
    def apply_user_id(self):
        return self.approval_info["ApplyUserId"]

    @property
    def apply_user_party(self):
        return self.approval_info["ApplyUserParty"]

    @property
    def apply_user_image(self):
        return self.approval_info["ApplyUserImage"]

    @property
    def approval_nodes(self):
        return self.approval_info["ApprovalNodes"]

    @property
    def notify_nodes(self):
        return self.approval_info["NotifyNodes"]

    @property
    def approver_step(self):
        return self.approval_info["approverstep"]


class TaskCardClickEvent(BaseEvent):
    """
    任务卡片事件推送

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/90240#%E4%BB%BB%E5%8A%A1%E5%8D%A1%E7%89%87%E4%BA%8B%E4%BB%B6%E6%8E%A8%E9%80%81
    """

    event = "taskcard_click"
    event_key = StringField("EventKey")
    agent = IntegerField("AgentID", 0)
    task_id = StringField("TaskId")


class ChangeExternalContactEvent(BaseEvent):
    """
    外部联系人事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/92130#%E6%B7%BB%E5%8A%A0%E5%A4%96%E9%83%A8%E8%81%94%E7%B3%BB%E4%BA%BA%E4%BA%8B%E4%BB%B6
    """

    event = "change_external_contact"
    change_type = StringField("ChangeType")
    welcome_code = StringField("WelcomeCode")
    state = StringField("State")
    user_id = StringField("UserID")
    external_user_id = StringField("ExternalUserID")
    del_source = StringField("Source")
    fail_reason = StringField("FailReason")


class ChangeExternalChatEvent(BaseEvent):
    """
    客户群变动事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/92130#%E6%B7%BB%E5%8A%A0%E5%A4%96%E9%83%A8%E8%81%94%E7%B3%BB%E4%BA%BA%E4%BA%8B%E4%BB%B6
    """

    event = "change_external_chat"
    change_type = StringField("ChangeType")
    update_detail = StringField("UpdateDetail")

    chat_id = StringField("ChatId")
    join_scene = StringField("JoinScene")
    quit_scene = StringField("QuitScene")
    mem_change_cnt = StringField("MemChangeCnt")


class ChangeExternalTagEvent(BaseEvent):
    """
    企业客户标签变动事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/92130#%E6%B7%BB%E5%8A%A0%E5%A4%96%E9%83%A8%E8%81%94%E7%B3%BB%E4%BA%BA%E4%BA%8B%E4%BB%B6
    """

    event = "change_external_tag"
    change_type = StringField("ChangeType")

    tag_id = StringField("Id")
    tag_type = StringField("TagType")


class SysApprovalChangeEvent(BaseEvent):
    """
    系统审批状态通知事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/91815
    """

    event = "sys_approval_change"
    agent = IntegerField("AgentID", 0)
    approval_info = BaseField("ApprovalInfo")

    @property
    def sp_no(self):
        return self.approval_info["SpNo"]

    @property
    def sp_name(self):
        return self.approval_info["SpName"]

    @property
    def template_id(self):
        return self.approval_info["TemplateId"]

    @property
    def sp_status(self):
        return self.approval_info["SpStatus"]

    @property
    def apply_time(self):
        return self.approval_info["ApplyTime"]

    @property
    def apply_user_id(self):
        return self.approval_info["Applyer"]["UserId"]

    @property
    def apply_user_party(self):
        return self.approval_info["Applyer"]["Party"]

    @property
    def sp_record(self):
        return self.approval_info["SpRecord"]

    @property
    def notifyer(self):
        return self.approval_info["Notifyer"]

    @property
    def comments(self):
        return self.approval_info["Comments"]

    @property
    def statu_change_event(self):
        return self.approval_info["StatuChangeEvent"]


class KFMsgOrEventEvent(BaseEvent):
    """
    微信客服消息事件

    详情请参阅
    https://work.weixin.qq.com/api/doc/90000/90135/94670
    """

    event = "kf_msg_or_event"
    token = StringField("Token")


class ModifyCalendarEvent(BaseEvent):
    """
    修改日历事件

    详情请参考
    https://developer.work.weixin.qq.com/document/path/93651
    """

    event = "modify_calendar"
    calendar_id = StringField("CalId")


class DeleteCalendarEvent(BaseEvent):
    """
    删除日历事件

    详情请参考
    https://developer.work.weixin.qq.com/document/path/93651
    """

    event = "delete_calendar"
    calendar_id = StringField("CalId")


class AddScheduleEvent(BaseEvent):
    """
    添加日程事件

    详情请参考
    https://developer.work.weixin.qq.com/document/path/93651
    """

    event = "add_schedule"
    calendar_id = StringField("CalId")
    schedule_id = StringField("ScheduleId")


class ModifyScheduleEvent(BaseEvent):
    """
    修改日程事件

    详情请参考
    https://developer.work.weixin.qq.com/document/path/93651
    """

    event = "modify_schedule"
    calendar_id = StringField("CalId")
    schedule_id = StringField("ScheduleId")


class DeleteScheduleEvent(BaseEvent):
    """
    删除日程事件

    详情请参考
    https://developer.work.weixin.qq.com/document/path/93651
    """

    event = "delete_schedule"
    calendar_id = StringField("CalId")
    schedule_id = StringField("ScheduleId")



# -*- coding: utf-8 -*-


import time

from wechatpy.client.base import BaseWeChatClient
from wechatpy.client import api


class WeChatClient(BaseWeChatClient):

    """
    微信 API 操作类
    通过这个类可以操作微信 API，发送主动消息、群发消息和创建自定义菜单等。
    """

    API_BASE_URL = "https://api.weixin.qq.com/cgi-bin/"

    card = api.WeChatCard()
    customservice = api.WeChatCustomService()
    datacube = api.WeChatDataCube()
    device = api.WeChatDevice()
    group = api.WeChatGroup()
    invoice = api.WeChatInvoice()
    jsapi = api.WeChatJSAPI()
    material = api.WeChatMaterial()
    media = api.WeChatMedia()
    menu = api.WeChatMenu()
    merchant = api.WeChatMerchant()
    message = api.WeChatMessage()
    misc = api.WeChatMisc()
    poi = api.WeChatPoi()
    qrcode = api.WeChatQRCode()
    scan = api.WeChatScan()
    semantic = api.WeChatSemantic()
    shakearound = api.WeChatShakeAround()
    tag = api.WeChatTag()
    template = api.WeChatTemplate()
    user = api.WeChatUser()
    wifi = api.WeChatWiFi()
    wxa = api.WeChatWxa()
    marketing = api.WeChatMarketing()
    cloud = api.WeChatCloud()

    def __init__(
        self,
        appid,
        secret,
        access_token=None,
        session=None,
        timeout=None,
        auto_retry=True,
    ):
        super().__init__(appid, access_token, session, timeout, auto_retry)
        self.appid = appid
        self.secret = secret

    def fetch_access_token(self):
        """
        获取 access token
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Basic_Information/Get_access_token.html

        :return: 返回的 JSON 数据包
        """
        return self._fetch_access_token(
            url="https://api.weixin.qq.com/cgi-bin/token",
            params={
                "grant_type": "client_credential",
                "appid": self.appid,
                "secret": self.secret,
            },
        )


class WeChatComponentClient(WeChatClient):

    """
    开放平台代公众号调用客户端
    """

    def __init__(
        self,
        appid,
        component,
        access_token=None,
        refresh_token=None,
        session=None,
        timeout=None,
    ):
        # 未用到secret，所以这里没有
        super().__init__(appid, "", "", session, timeout)
        self.appid = appid
        self.component = component
        # 如果公众号是刚授权，外部还没有缓存access_token和refresh_token
        # 可以传入这两个值，session 会缓存起来。
        # 如果外部已经缓存，这里只需要传入 appid，component和session即可
        cache_access_token = self.session.get(self.access_token_key)

        if access_token and (not cache_access_token or cache_access_token != access_token):
            self.session.set(self.access_token_key, access_token, 7200)
        if refresh_token:
            self.session.set(self.refresh_token_key, refresh_token)

    @property
    def access_token_key(self):
        return f"{self.appid}_access_token"

    @property
    def refresh_token_key(self):
        return f"{self.appid}_refresh_token"

    @property
    def access_token(self):
        access_token = self.session.get(self.access_token_key)
        if not access_token:
            self.fetch_access_token()
            access_token = self.session.get(self.access_token_key)
        return access_token

    @property
    def refresh_token(self):
        return self.session.get(self.refresh_token_key)

    def fetch_access_token(self):
        """
        获取 access token
        详情请参考 https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list\
        &t=resource/res_list&verify=1&id=open1419318587&token=&lang=zh_CN

        这是内部刷新机制。请不要完全依赖！
        因为有可能在缓存期间没有对此公众号的操作，造成refresh_token失效。

        :return: 返回的 JSON 数据包
        """
        expires_in = 7200
        result = self.component.refresh_authorizer_token(self.appid, self.refresh_token)
        if "expires_in" in result:
            expires_in = result["expires_in"]
        self.session.set(self.access_token_key, result["authorizer_access_token"], expires_in)
        self.expires_at = int(time.time()) + expires_in
        return result



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI


class MerchantCategory(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def get_sub_categories(self, cate_id):
        res = self._post(
            "merchant/category/getsub",
            data={"cate_id": cate_id},
            result_processor=lambda x: x["cate_list"],
        )
        return res

    def get_sku_list(self, cate_id):
        res = self._post(
            "merchant/category/getsku",
            data={"cate_id": cate_id},
            result_processor=lambda x: x["sku_table"],
        )
        return res

    def get_properties(self, cate_id):
        res = self._post(
            "merchant/category/getproperty",
            data={"cate_id": cate_id},
            result_processor=lambda x: x["properties"],
        )
        return res



# -*- coding: utf-8 -*-

import time
from datetime import datetime

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatShakeAround(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    @classmethod
    def _to_timestamp(cls, date):
        if isinstance(date, str):
            date = datetime.strptime(date, "%Y-%m-%d %H:%M:%S")
        if isinstance(date, datetime):
            timestamp = int(time.mktime(date.timetuple()))
            return timestamp
        return int(date)

    def apply_device_id(self, quantity, reason, poi_id=None, comment=None):
        """
        申请设备ID
        详情请参考
        http://mp.weixin.qq.com/wiki/15/b9e012f917e3484b7ed02771156411f3.html

        :param quantity: 申请的设备ID的数量，单次新增设备超过500个，需走人工审核流程
        :param reason: 申请理由，不超过100个字
        :param poi_id: 可选，设备关联的门店ID
        :param comment: 可选，备注，不超过15个汉字或30个英文字母
        :return: 申请的设备信息
        """
        data = optionaldict()
        data["quantity"] = quantity
        data["apply_reason"] = reason
        data["poi_id"] = poi_id
        data["comment"] = comment
        res = self._post(
            "shakearound/device/applyid",
            data=data,
            result_processor=lambda x: x["data"],
        )
        return res

    def update_device(self, device_id=None, uuid=None, major=None, minor=None, comment=None):
        """
        更新设备信息
        详情请参考
        http://mp.weixin.qq.com/wiki/15/b9e012f917e3484b7ed02771156411f3.html

        :param device_id: 设备编号，若填了UUID、major、minor，则可不填设备编号，若二者都填，则以设备编号为优先
        :param uuid: UUID
        :param major: major
        :param minor: minor
        :param comment: 设备的备注信息，不超过15个汉字或30个英文字母。
        :return: 返回的 JSON 数据包
        """
        data = optionaldict()
        data["comment"] = comment
        data["device_identifier"] = {
            "device_id": device_id,
            "uuid": uuid,
            "major": major,
            "minor": minor,
        }
        return self._post("shakearound/device/update", data=data)

    def bind_device_location(self, poi_id, device_id=None, uuid=None, major=None, minor=None):
        """
        配置设备与门店的关联关系
        详情请参考
        http://mp.weixin.qq.com/wiki/15/b9e012f917e3484b7ed02771156411f3.html

        :param poi_id: 待关联的门店ID
        :param device_id: 设备编号，若填了UUID、major、minor，则可不填设备编号，若二者都填，则以设备编号为优先
        :param uuid: UUID
        :param major: major
        :param minor: minor
        :return: 返回的 JSON 数据包
        """
        data = optionaldict()
        data["poi_id"] = poi_id
        data["device_identifier"] = {
            "device_id": device_id,
            "uuid": uuid,
            "major": major,
            "minor": minor,
        }
        return self._post("shakearound/device/bindlocation", data=data)

    def search_device(self, identifiers=None, apply_id=None, begin=0, count=10):
        """
        查询设备列表
        详情请参考
        http://mp.weixin.qq.com/wiki/15/b9e012f917e3484b7ed02771156411f3.html

        :param identifiers: 设备 ID 信息列表
        :param apply_id: 批次ID，申请设备ID超出500个时所返回批次ID
        :param begin: 设备列表的起始索引值
        :param count: 待查询的设备个数
        :return: 设备列表
        """
        data = optionaldict()
        data["begin"] = begin
        data["count"] = count
        data["apply_id"] = apply_id
        if identifiers:
            data["device_identifiers"] = identifiers
        res = self._post("shakearound/device/search", data=data, result_processor=lambda x: x["data"])
        return res

    def add_page(self, title, description, icon_url, page_url, comment=None):
        """
        新增页面
        详情请参考
        http://mp.weixin.qq.com/wiki/5/6626199ea8757c752046d8e46cf13251.html

        :param title: 在摇一摇页面展示的主标题，不超过6个字
        :param description: 在摇一摇页面展示的副标题，不超过7个字
        :param icon_url: 在摇一摇页面展示的图片。图片需先上传至微信侧服务器，
                        用“素材管理-上传图片素材”接口上传图片，返回的图片URL再配置在此处
        :param page_url: 跳转链接
        :param comment: 可选，页面的备注信息，不超过15个字
        :return: 页面信息
        """
        data = optionaldict()
        data["title"] = title
        data["description"] = description
        data["icon_url"] = icon_url
        data["page_url"] = page_url
        data["comment"] = comment
        res = self._post("shakearound/page/add", data=data, result_processor=lambda x: x["data"])
        return res

    def update_page(self, page_id, title, description, icon_url, page_url, comment=None):
        """
        编辑页面信息
        详情请参考
        http://mp.weixin.qq.com/wiki/5/6626199ea8757c752046d8e46cf13251.html

        :param page_id: 摇周边页面唯一ID
        :param title: 在摇一摇页面展示的主标题，不超过6个字
        :param description: 在摇一摇页面展示的副标题，不超过7个字
        :param icon_url: 在摇一摇页面展示的图片。图片需先上传至微信侧服务器，
                        用“素材管理-上传图片素材”接口上传图片，返回的图片URL再配置在此处
        :param page_url: 跳转链接
        :param comment: 可选，页面的备注信息，不超过15个字
        :return: 页面信息
        """
        data = optionaldict()
        data["page_id"] = page_id
        data["title"] = title
        data["description"] = description
        data["icon_url"] = icon_url
        data["page_url"] = page_url
        data["comment"] = comment
        res = self._post("shakearound/page/update", data=data, result_processor=lambda x: x["data"])
        return res

    def search_pages(self, page_ids=None, begin=0, count=10):
        """
        查询页面列表
        详情请参考
        http://mp.weixin.qq.com/wiki/5/6626199ea8757c752046d8e46cf13251.html

        :param page_ids: 指定页面的id列表
        :param begin: 页面列表的起始索引值
        :param count: 待查询的页面个数
        :return: 页面查询结果信息
        """
        if not page_ids:
            data = {"type": 2, "begin": begin, "count": count}
        else:
            if not isinstance(page_ids, (tuple, list)):
                page_ids = [page_ids]
            data = {"type": 1, "page_ids": page_ids}

        res = self._post("shakearound/page/search", data=data, result_processor=lambda x: x["data"])
        return res

    def delete_page(self, page_id):
        """
        删除页面
        详情请参考
        http://mp.weixin.qq.com/wiki/5/6626199ea8757c752046d8e46cf13251.html

        :param page_id: 指定页面的id列表
        :return: 返回的 JSON 数据包
        """
        return self._post("shakearound/page/delete", data={"page_id": page_id})

    def add_material(self, media_file, media_type="icon"):
        """
        上传图片素材
        详情请参考
        http://mp.weixin.qq.com/wiki/5/e997428269ff189d8f9a4b9e177be2d9.html

        :param media_file: 要上传的文件，一个 File-object
        :param media_type: 摇一摇素材类型, 取值为 icon或者 license, 默认 icon.
        :return: 上传的素材信息
        """
        res = self._post(
            "shakearound/material/add",
            files={"media": media_file},
            params={"type": media_type},
            result_processor=lambda x: x["data"],
        )
        return res

    def bind_device_pages(self, page_ids, bind, append, device_id=None, uuid=None, major=None, minor=None):
        """
        配置设备与页面的关联关系
        详情请参考
        http://mp.weixin.qq.com/wiki/12/c8120214ec0ba08af5dfcc0da1a11400.html

        :param page_ids: 待关联的页面列表
        :param bind: 关联操作标志位， 0为解除关联关系，1为建立关联关系
        :param append: 新增操作标志位， 0为覆盖，1为新增
        :param device_id: 设备编号，若填了UUID、major、minor，则可不填设备编号，若二者都填，则以设备编号为优先
        :param uuid: UUID
        :param major: major
        :param minor: minor
        :return: 返回的 JSON 数据包
        """
        if not isinstance(page_ids, (tuple, list)):
            page_ids = [page_ids]
        data = {
            "page_ids": page_ids,
            "bind": int(bind),
            "append": int(append),
            "device_identifier": {
                "device_id": device_id,
                "uuid": uuid,
                "major": major,
                "minor": minor,
            },
        }
        return self._post("shakearound/device/bindpage", data=data)

    def get_shake_info(self, ticket):
        """
        获取摇周边的设备及用户信息
        详情请参考
        http://mp.weixin.qq.com/wiki/3/34904a5db3d0ec7bb5306335b8da1faf.html

        :param ticket: 摇周边业务的ticket，可在摇到的URL中得到，ticket生效时间为30分钟
        :return: 设备及用户信息
        """
        res = self._post(
            "shakearound/user/getshakeinfo",
            data={"ticket": ticket},
            result_processor=lambda x: x["data"],
        )
        return res

    def get_device_statistics(self, begin_date, end_date, device_id=None, uuid=None, major=None, minor=None):
        """
        以设备为维度的数据统计接口
        http://mp.weixin.qq.com/wiki/0/8a24bcacad40fe7ee98d1573cb8a6764.html

        :param begin_date: 起始时间，最长时间跨度为30天
        :param end_date: 结束时间，最长时间跨度为30天
        :param device_id: 设备编号，若填了UUID、major、minor，则可不填设备编号，若二者都填，则以设备编号为优先
        :param uuid: UUID
        :param major: major
        :param minor: minor
        """
        data = {
            "device_identifier": {
                "device_id": device_id,
                "uuid": uuid,
                "major": major,
                "minor": minor,
            },
            "begin_date": self._to_timestamp(begin_date),
            "end_date": self._to_timestamp(end_date),
        }
        res = self._post(
            "shakearound/statistics/device",
            data=data,
            result_processor=lambda x: x["data"],
        )
        return res

    def get_page_statistics(self, page_id, begin_date, end_date):
        """
        以页面为维度的数据统计接口
        详情请参考
        http://mp.weixin.qq.com/wiki/0/8a24bcacad40fe7ee98d1573cb8a6764.html

        :param page_id: 页面 ID
        :param begin_date: 起始时间，最长时间跨度为30天
        :param end_date: 结束时间，最长时间跨度为30天
        :return: 统计数据
        """
        res = self._post(
            "shakearound/statistics/page",
            data={
                "page_id": page_id,
                "begin_date": self._to_timestamp(begin_date),
                "end_date": self._to_timestamp(end_date),
            },
            result_processor=lambda x: x["data"],
        )
        return res

    def get_apply_status(self, apply_id):
        """
        查询设备ID申请审核状态
        详情请参考
        http://mp.weixin.qq.com/wiki/15/b9e012f917e3484b7ed02771156411f3.html

        :param apply_id: 批次ID，申请设备ID时所返回的批次ID
        :return: 批次状态信息
        """
        res = self._post(
            "shakearound/device/applystatus",
            data={
                "apply_id": apply_id,
            },
            result_processor=lambda x: x["data"],
        )
        return res



# -*- coding: utf-8 -*-


import operator as op

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatSchedule(BaseWeChatAPI):
    def add(
        self,
        organizer,
        start_time,
        end_time,
        attendees=(),
        summary="",
        description="",
        is_remind=True,
        remind_before_event_secs=3600,
        is_repeat=False,
        repeat_type=0,
        location="",
        calendar_id="",
    ):
        """
        创建日程

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93648#%E5%88%9B%E5%BB%BA%E6%97%A5%E7%A8%8B

        :param organizer: 组织者
        :param start_time: 日程开始时间，Unix时间戳
        :param end_time: 日程结束时间，Unix时间戳
        :param attendees: 日程参与者列表。最多支持2000人
        :param summary: 日程标题。0 ~ 128 字符。不填会默认显示为“新建事件”
        :param description: 日程描述。0 ~ 512 字符
        :param is_remind: 是否需要提醒
        :param remind_before_event_secs: 日程开始（start_time）前多少秒提醒，当is_remind为1时有效
        :param is_repeat: 是否重复日程
        :param repeat_type: 重复类型，当is_repeat为1时有效。目前支持如下类型：
            0 - 每日
            1 - 每周
            2 - 每月
            5 - 每年
            7 - 工作日
        :param location: 日程地址。0 ~ 128 字符
        :param calendar_id: 日程所属日历ID。注意，这个日历必须是属于组织者(organizer)的日历；如果不填，那么插入到组织者的默认日历上

        :type organizer: str
        :type start_time: int
        :type end_time: int
        :type attendees: list[str]
        :type summary: str
        :type description: str
        :type is_remind: bool
        :type remind_before_event_secs: int
        :type is_repeat: bool
        :type repeat_type: int
        :type location: str
        :type calendar_id: str

        :return: 日程ID
        :rtype: str
        """

        data = {
            "schedule": {
                "organizer": organizer,
                "start_time": start_time,
                "end_time": end_time,
                "attendees": [{"userid": userid} for userid in attendees],
                "summary": summary,
                "description": description,
                "reminders": {
                    "is_remind": int(is_remind),
                    "remind_before_event_secs": remind_before_event_secs,
                    "is_repeat": int(is_repeat),
                    "repeat_type": repeat_type,
                },
                "location": location,
                "cal_id": calendar_id,
            }
        }
        return self._post("oa/schedule/add", data=data, result_processor=op.itemgetter("schedule_id"))

    def update(
        self,
        organizer,
        schedule_id,
        start_time,
        end_time,
        attendees=(),
        summary="",
        description="",
        is_remind=True,
        remind_before_event_secs=3600,
        is_repeat=False,
        repeat_type=0,
        location="",
        calendar_id="",
    ):
        """
        更新日程

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93648#%E6%9B%B4%E6%96%B0%E6%97%A5%E7%A8%8B

        :param organizer: 组织者
        :param schedule_id: 日程ID
        :param start_time: 日程开始时间，Unix时间戳
        :param end_time: 日程结束时间，Unix时间戳
        :param attendees: 日程参与者列表。最多支持2000人
        :param summary: 日程标题。0 ~ 128 字符。不填会默认显示为“新建事件”
        :param description: 日程描述。0 ~ 512 字符
        :param is_remind: 是否需要提醒
        :param remind_before_event_secs: 日程开始（start_time）前多少秒提醒，当is_remind为1时有效
        :param is_repeat: 是否重复日程
        :param repeat_type: 重复类型，当is_repeat为1时有效。目前支持如下类型：
            0 - 每日
            1 - 每周
            2 - 每月
            5 - 每年
            7 - 工作日
        :param location: 日程地址。0 ~ 128 字符
        :param calendar_id: 日程所属日历ID。注意，这个日历必须是属于组织者(organizer)的日历；如果不填，那么插入到组织者的默认日历上

        :type organizer: str
        :type schedule_id: str
        :type start_time: int
        :type end_time: int
        :type attendees: list[str]
        :type summary: str
        :type description: str
        :type is_remind: bool
        :type remind_before_event_secs: int
        :type is_repeat: bool
        :type repeat_type: int
        :type location: str
        :type calendar_id: str
        """

        data = {
            "schedule": {
                "organizer": organizer,
                "schedule_id": schedule_id,
                "start_time": start_time,
                "end_time": end_time,
                "attendees": [{"userid": userid} for userid in attendees],
                "summary": summary,
                "description": description,
                "reminders": {
                    "is_remind": int(is_remind),
                    "remind_before_event_secs": remind_before_event_secs,
                    "is_repeat": int(is_repeat),
                    "repeat_type": repeat_type,
                },
                "location": location,
                "cal_id": calendar_id,
            }
        }
        return self._post("oa/schedule/update", data=data)

    def get(self, schedule_ids):
        """
        获取日程

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93648#%E8%8E%B7%E5%8F%96%E6%97%A5%E7%A8%8B%E8%AF%A6%E6%83%85

        :param schedule_ids: 日程ID列表。一次最多可获取1000条
        :type schedule_ids: list[str]

        :return: 日程列表
        :rtype: list[dict]
        """
        return self._post(
            "oa/schedule/get",
            data={"schedule_id_list": schedule_ids},
            result_processor=op.itemgetter("schedule_list"),
        )

    def delete(self, schedule_id):
        """
        取消日程（删除日程）

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93648#%E5%8F%96%E6%B6%88%E6%97%A5%E7%A8%8B

        :param schedule_id: 日程ID
        """
        return self._post("oa/schedule/del", data={"schedule_id": schedule_id})

    def get_by_calendar(self, calendar_id, offset=0, limit=500):
        """
        获取日历下的日程列表

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93648#%E8%8E%B7%E5%8F%96%E6%97%A5%E5%8E%86%E4%B8%8B%E7%9A%84%E6%97%A5%E7%A8%8B%E5%88%97%E8%A1%A8
        （注意，被取消的日程也可以拉取详情，调用者需要检查status）

        :param calendar_id: 日历ID
        :param offset: 分页，偏移量, 默认为0
        :param limit: 分页，预期请求的数据量，默认为500，取值范围 1 ~ 1000

        :return: 日程列表
        :rtype: list[dict]
        """
        return self._post(
            "oa/schedule/get_by_calendar",
            data={"cal_id": calendar_id, "offset": offset, "limit": limit},
            result_processor=op.itemgetter("schedule_list"),
        )

    def del_users(self, schedule_id, users):
        """
        删除日程参与者

        详情请参考
        https://developer.work.weixin.qq.com/document/path/93648#%E5%88%A0%E9%99%A4%E6%97%A5%E7%A8%8B%E5%8F%82%E4%B8%8E%E8%80%85

        :param schedule_id: 日程 ID
        :param users: 日程参与者 ID 列表

        :type users: list[str]
        """
        attendees = [{"userid": i for i in users}]
        return self._post("oa/schedule/del_attendees", data={"schedule_id": schedule_id, "attendees": attendees})



# -*- coding: utf-8 -*-

import inspect
import logging

import requests
import xmltodict
from xml.parsers.expat import ExpatError
from optionaldict import optionaldict

from wechatpy.crypto import WeChatRefundCrypto
from wechatpy.utils import random_string
from wechatpy.exceptions import WeChatPayException, InvalidSignatureException
from wechatpy.pay.utils import (
    calculate_signature,
    calculate_signature_hmac,
    _check_signature,
    dict_to_xml,
)
from wechatpy.pay.api.base import BaseWeChatPayAPI
from wechatpy.pay import api

logger = logging.getLogger(__name__)


def _is_api_endpoint(obj):
    return isinstance(obj, BaseWeChatPayAPI)


class WeChatPay:
    """
    微信支付接口

    :param appid: 微信公众号 appid
    :param sub_appid: 当前调起支付的小程序APPID
    :param api_key: 商户 key,不要在这里使用小程序的密钥
    :param mch_id: 商户号
    :param sub_mch_id: 可选，子商户号，受理模式下必填
    :param mch_cert: 必填，商户证书路径
    :param mch_key: 必填，商户证书私钥路径
    :param timeout: 可选，请求超时时间，单位秒，默认无超时设置
    :param sandbox: 可选，是否使用测试环境，默认为 False
    """

    redpack = api.WeChatRedpack()
    """红包接口"""
    transfer = api.WeChatTransfer()
    """企业付款接口"""
    coupon = api.WeChatCoupon()
    """代金券接口"""
    order = api.WeChatOrder()
    """订单接口"""
    refund = api.WeChatRefund()
    """退款接口"""
    micropay = api.WeChatMicroPay()
    """刷卡支付接口"""
    tools = api.WeChatTools()
    """工具类接口"""
    jsapi = api.WeChatJSAPI()
    """公众号网页 JS 支付接口"""
    withhold = api.WeChatWithhold()
    """代扣接口"""
    app_auth = api.WeChatAppAuth()
    """实名认证接口"""

    API_BASE_URL = "https://api.mch.weixin.qq.com/"

    def __new__(cls, *args, **kwargs):
        self = super().__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
        for name, _api in api_endpoints:
            api_cls = type(_api)
            _api = api_cls(self)
            setattr(self, name, _api)
        return self

    def __init__(
        self,
        appid,
        api_key,
        mch_id,
        sub_mch_id=None,
        mch_cert=None,
        mch_key=None,
        timeout=None,
        sandbox=False,
        sub_appid=None,
    ):
        self.appid = appid
        self.sub_appid = sub_appid
        self.api_key = api_key
        self.mch_id = mch_id
        self.sub_mch_id = sub_mch_id
        self.mch_cert = mch_cert
        self.mch_key = mch_key
        self._using_pkcs12_cert = False
        self.timeout = timeout
        self.sandbox = sandbox
        self._sandbox_api_key = None
        self._http = requests.Session()
        if mch_cert and mch_cert.endswith(".p12"):
            from requests_pkcs12 import Pkcs12Adapter

            # 商户 .p12 格式证书，证书密码默认为商户 ID
            self._http.mount(
                self.API_BASE_URL, Pkcs12Adapter(pkcs12_filename=self.mch_cert, pkcs12_password=self.mch_id)
            )
            self._using_pkcs12_cert = True

    def _fetch_sandbox_api_key(self):
        nonce_str = random_string(32)
        sign = calculate_signature({"mch_id": self.mch_id, "nonce_str": nonce_str}, self.api_key)
        payload = dict_to_xml(
            {
                "mch_id": self.mch_id,
                "nonce_str": nonce_str,
            },
            sign=sign,
        )
        headers = {"Content-Type": "text/xml"}
        api_url = f"{self.API_BASE_URL}sandboxnew/pay/getsignkey"
        response = self._http.post(api_url, data=payload, headers=headers)
        return xmltodict.parse(response.text)["xml"].get("sandbox_signkey")

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(("http://", "https://")):
            api_base_url = kwargs.pop("api_base_url", self.API_BASE_URL)
            if self.sandbox:
                api_base_url = f"{api_base_url}sandboxnew/"
            url = f"{api_base_url}{url_or_endpoint}"
        else:
            url = url_or_endpoint

        if isinstance(kwargs.get("data", ""), dict):
            data = kwargs["data"]
            if "mchid" not in data:
                # Fuck Tencent
                data.setdefault("mch_id", self.mch_id)
            data.setdefault("sub_mch_id", self.sub_mch_id)
            data.setdefault("nonce_str", random_string(32))
            data = optionaldict(data)

            if data.get("sign_type", "MD5") == "HMAC-SHA256":
                sign = calculate_signature_hmac(data, self.sandbox_api_key if self.sandbox else self.api_key)
            else:
                sign = calculate_signature(data, self.sandbox_api_key if self.sandbox else self.api_key)
            body = dict_to_xml(data, sign)
            body = body.encode("utf-8")
            kwargs["data"] = body

        # 商户 PEM 证书
        if not self._using_pkcs12_cert and self.mch_cert and self.mch_key:
            kwargs["cert"] = (self.mch_cert, self.mch_key)

        kwargs["timeout"] = kwargs.get("timeout", self.timeout)
        logger.debug("Request to WeChat API: %s %s\n%s", method, url, kwargs)
        res = self._http.request(method=method, url=url, **kwargs)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatPayException(
                return_code=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )

        return self._handle_result(res)

    def _handle_result(self, res):
        res.encoding = "utf-8-sig"
        xml = res.text
        logger.debug("Response from WeChat API \n %s", xml)
        try:
            data = xmltodict.parse(xml)["xml"]
        except (xmltodict.ParsingInterrupted, ExpatError):
            # 解析 XML 失败
            logger.debug("WeChat payment result xml parsing error", exc_info=True)
            return xml

        return_code = data["return_code"]
        return_msg = data.get("return_msg", data.get("retmsg"))
        result_code = data.get("result_code", data.get("retcode"))
        errcode = data.get("err_code")
        errmsg = data.get("err_code_des")
        if return_code != "SUCCESS" or result_code != "SUCCESS":
            # 返回状态码不为成功
            raise WeChatPayException(
                return_code,
                result_code,
                return_msg,
                errcode,
                errmsg,
                client=self,
                request=res.request,
                response=res,
            )
        return data

    def get(self, url, **kwargs):
        return self._request(method="get", url_or_endpoint=url, **kwargs)

    def post(self, url, **kwargs):
        return self._request(method="post", url_or_endpoint=url, **kwargs)

    def check_signature(self, params):
        return _check_signature(params, self.api_key if not self.sandbox else self.sandbox_api_key)

    @classmethod
    def get_payment_data(cls, xml):
        """
        解析微信支付结果通知，获得appid, mch_id, out_trade_no, transaction_id
        如果你需要进一步判断，请先用appid, mch_id来生成WeChatPay,
        然后用`wechatpay.parse_payment_result(xml)`来校验支付结果

        使用示例::

            from wechatpy.pay import WeChatPay
            # 假设你已经获取了微信服务器推送的请求中的xml数据并存入xml变量
            data = WeChatPay.get_payment_appid(xml)
            {
                "appid": "公众号或者小程序的id",
                "mch_id": "商户id",
            }

        """
        try:
            data = xmltodict.parse(xml)
        except (xmltodict.ParsingInterrupted, ExpatError):
            raise ValueError("invalid xml")
        if not data or "xml" not in data:
            raise ValueError("invalid xml")
        data = data["xml"]
        return {
            "appid": data["appid"],
            "mch_id": data["mch_id"],
            "out_trade_no": data["out_trade_no"],
            "transaction_id": data["transaction_id"],
        }

    def parse_payment_result(self, xml):
        """解析微信支付结果通知"""
        try:
            data = xmltodict.parse(xml)
        except (xmltodict.ParsingInterrupted, ExpatError):
            raise InvalidSignatureException()

        if not data or "xml" not in data:
            raise InvalidSignatureException()

        data = data["xml"]
        sign = data.pop("sign", None)
        real_sign = calculate_signature(data, self.api_key if not self.sandbox else self.sandbox_api_key)
        if sign != real_sign:
            raise InvalidSignatureException()

        for key in (
            "total_fee",
            "settlement_total_fee",
            "cash_fee",
            "coupon_fee",
            "coupon_count",
        ):
            if key in data:
                data[key] = int(data[key])
        data["sign"] = sign
        return data

    def parse_refund_notify_result(self, xml):
        """解析微信退款结果通知"""
        refund_crypto = WeChatRefundCrypto(self.api_key if not self.sandbox else self.sandbox_api_key)
        data = refund_crypto.decrypt_message(xml, self.appid, self.mch_id)
        for key in (
            "total_fee",
            "settlement_total_fee",
            "refund_fee",
            "settlement_refund_fee",
        ):
            if key in data:
                data[key] = int(data[key])
        return data

    @property
    def sandbox_api_key(self):
        if self.sandbox and self._sandbox_api_key is None:
            self._sandbox_api_key = self._fetch_sandbox_api_key()

        return self._sandbox_api_key



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI


class MerchantGroup(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def add(self, name, product_list):
        return self._post(
            "merchant/group/add",
            data={"group_detail": {"group_name": name, "product_list": product_list}},
        )

    def delete(self, group_id):
        return self._post("merchant/group/del", data={"group_id": group_id})

    def update(self, group_id, name):
        return self._post(
            "merchant/group/propertymod",
            data={"group_id": group_id, "group_name": name},
        )

    def update_product(self, group_id, product):
        return self._post("merchant/group/productmod", data={"group_id": group_id, "product": product})

    def get_all(self):
        res = self._get("merchant/group/getall", result_processor=lambda x: x["groups_detail"])
        return res

    def get(self, group_id):
        res = self._post(
            "merchant/group/getbyid",
            data={"group_id": group_id},
            result_processor=lambda x: x["group_detail"],
        )
        return res



# -*- coding: utf-8 -*-

import hashlib
import time
import datetime
from operator import itemgetter
from urllib.parse import quote

from optionaldict import optionaldict
from wechatpy.utils import to_binary
from wechatpy.client.api.base import BaseWeChatAPI


class WeChatCustomService(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/customservice/"

    def add_account(self, account, nickname, password):
        """
        添加客服账号
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Service_Center_messages.html#%E6%B7%BB%E5%8A%A0%E5%AE%A2%E6%9C%8D%E5%B8%90%E5%8F%B7

        :param account: 完整客服账号，格式为：账号前缀@公众号微信号
        :param nickname: 客服昵称，最长6个汉字或12个英文字符
        :param password: 客服账号登录密码
        :return: 返回的 JSON 数据包
        """
        password = to_binary(password)
        password = hashlib.md5(password).hexdigest()
        return self._post(
            "kfaccount/add",
            data={"kf_account": account, "nickname": nickname, "password": password},
        )

    def update_account(self, account, nickname, password):
        """
        修改客服账号
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Service_Center_messages.html#%E4%BF%AE%E6%94%B9%E5%AE%A2%E6%9C%8D%E5%B8%90%E5%8F%B7

        :param account: 完整客服账号，格式为：账号前缀@公众号微信号
        :param nickname: 客服昵称，最长6个汉字或12个英文字符
        :param password: 客服账号登录密码
        :return: 返回的 JSON 数据包
        """
        password = to_binary(password)
        password = hashlib.md5(password).hexdigest()
        return self._post(
            "kfaccount/update",
            data={"kf_account": account, "nickname": nickname, "password": password},
        )

    def delete_account(self, account):
        """
        删除客服账号
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Service_Center_messages.html#%E5%88%A0%E9%99%A4%E5%AE%A2%E6%9C%8D%E5%B8%90%E5%8F%B7

        :param account: 完整客服账号，格式为：账号前缀@公众号微信号
        :return: 返回的 JSON 数据包
        """
        params_data = [
            f"access_token={quote(self.access_token)}",
            f"kf_account={quote(to_binary(account), safe=b'/@')}",
        ]
        params = "&".join(params_data)
        return self._get("kfaccount/del", params=params)

    def get_accounts(self):
        """
        获取所有客服账号
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Service_Center_messages.html#%E8%8E%B7%E5%8F%96%E6%89%80%E6%9C%89%E5%AE%A2%E6%9C%8D%E8%B4%A6%E5%8F%B7

        :return: 客服账号列表
        """
        res = self._get("getkflist", result_processor=itemgetter("kf_list"))
        return res

    def upload_headimg(self, account, media_file):
        """
        设置客服帐号的头像
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Service_Center_messages.html#%E8%AE%BE%E7%BD%AE%E5%AE%A2%E6%9C%8D%E5%B8%90%E5%8F%B7%E7%9A%84%E5%A4%B4%E5%83%8F

        :param account: 完整客服帐号，格式为：帐号前缀@公众号微信号
        :param media_file: 要上传的头像文件，一个 File-Object
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "kfaccount/uploadheadimg",
            params={"kf_account": account},
            files={"media": media_file},
        )

    def get_online_accounts(self):
        """
        获取在线客服接待信息
        详情请参考
        http://mp.weixin.qq.com/wiki/9/6fff6f191ef92c126b043ada035cc935.html

        :return: 客服接待信息列表
        """
        res = self._get(
            "getonlinekflist",
            result_processor=itemgetter("kf_online_list"),
        )
        return res

    def create_session(self, openid, account, text=None):
        """
        多客服创建会话
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Customer_Service/Session_control.html

        :param openid: 客户 openid
        :param account: 完整客服帐号，格式为：帐号前缀@公众号微信号
        :param text: 附加信息，可选
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(openid=openid, kf_account=account, text=text)
        return self._post("kfsession/create", data=data)

    def close_session(self, openid, account, text=None):
        """
        多客服关闭会话
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Customer_Service/Session_control.html

        :param openid: 客户 openid
        :param account: 完整客服帐号，格式为：帐号前缀@公众号微信号
        :param text: 附加信息，可选
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(openid=openid, kf_account=account, text=text)
        return self._post("kfsession/close", data=data)

    def get_session(self, openid):
        """
        获取客户的会话状态，如果不存在，则 kf_account 为空
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Customer_Service/Session_control.html

        :param openid: 粉丝的 openid
        :return: 返回的 JSON 数据包
        """
        return self._get(
            "kfsession/getsession",
            params={"openid": openid},
        )

    def get_session_list(self, account):
        """
        获取客服的会话列表
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Customer_Service/Session_control.html

        :param account: 完整客服帐号，格式为：帐号前缀@公众号微信号
        :return: 客服的会话列表
        """
        res = self._get(
            "kfsession/getsessionlist",
            params={"kf_account": account},
            result_processor=itemgetter("sessionlist"),
        )
        return res

    def get_wait_case(self):
        """
        获取未接入会话列表
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Customer_Service/Session_control.html

        :return: 返回的 JSON 数据包
        """
        return self._get("kfsession/getwaitcase")

    def get_records(self, start_time, end_time, msgid=1, number=10000):
        """
        获取客服聊天记录
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Customer_Service/Obtain_chat_transcript.html

        :param start_time: 查询开始时间，UNIX 时间戳
        :param end_time: 查询结束时间，UNIX 时间戳，每次查询不能跨日查询
        :param msgid: 消息id顺序从小到大，从1开始
        :param number: 每次获取条数，最多10000条

        :return: 返回的 JSON 数据包
        """
        if isinstance(start_time, datetime.datetime):
            start_time = time.mktime(start_time.timetuple())
        if isinstance(end_time, datetime.datetime):
            end_time = time.mktime(end_time.timetuple())
        record_data = {
            "starttime": int(start_time),
            "endtime": int(end_time),
            "msgid": msgid,
            "number": number,
        }
        res = self._post(
            "msgrecord/getmsglist",
            data=record_data,
        )
        return res



# -*- coding: utf-8 -*-

from wechatpy.session import SessionStorage


class MemoryStorage(SessionStorage):
    def __init__(self):
        self._data = {}

    def get(self, key, default=None):
        return self._data.get(key, default)

    def set(self, key, value, ttl=None):
        if value is None:
            return
        self._data[key] = value

    def delete(self, key):
        self._data.pop(key, None)



from wechatpy.client.api.base import BaseWeChatAPI


class WeChatDraft(BaseWeChatAPI):
    def add(self, articles):
        """
        新增草稿素材, 原 新增永久图文素材
        用于替换原本的新增永久图文素材 add_articles 方法
        详情参考
        https://developers.weixin.qq.com/doc/offiaccount/Draft_Box/Add_draft.html

        :param articles: 图文素材数组
        :type articles: list[dict]
        :return: 返回的 JSON 数据包
        """
        articles_data = []
        for article in articles:
            articles_data.append(
                {
                    "title": article["title"],
                    "author": article.get("author", ""),
                    "digest": article.get("digest", ""),
                    "content": article["content"],
                    "content_source_url": article.get("content_source_url", ""),
                    "thumb_media_id": article["thumb_media_id"],
                    "need_open_comment": int(article.get("need_open_comment", False)),
                    "only_fans_can_comment": int(article.get("only_fans_can_comment", False)),
                }
            )
        return self._post("draft/add", data={"articles": articles_data})

    def get(self, media_id: str) -> dict:
        """
        获取草稿
        新增草稿后，开发者可以根据草稿指定的字段来下载草稿。
        详情参考
        https://developers.weixin.qq.com/doc/offiaccount/Draft_Box/Get_draft.html

        :param media_id: 要获取的草稿的 media_id
        :type media_id: str
        :return: 返回的 JSON 数据包
        """
        return self._post("draft/get", data={"media_id": media_id})

    def delete(self, media_id: str) -> dict:
        """
        删除草稿
        新增草稿后，开发者可以根据本接口来删除不再需要的草稿，节省空间。此操作无法撤销，请谨慎操作。
        详情参考
        https://developers.weixin.qq.com/doc/offiaccount/Draft_Box/Delete_draft.html

        :param media_id: 要删除的草稿的 media_id
        :type media_id: str
        :return: 返回的 JSON 数据包
        """
        return self._post("draft/delete", data={"media_id": media_id})

    def update(self, media_id, index, articles) -> dict:
        """
        修改草稿
        开发者可通过本接口对草稿进行修改。
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Draft_Box/Update_draft.html

        :param media_id: 要修改的图文消息的 id
        :param index: 要更新的文章在图文消息中的位置（多图文消息时，此字段才有意义），第一篇为 0
        :param articles: 草稿内容，详情见链接
        :return: 返回的 JSON 数据包
        """
        return self._post("draft/update", data={"media_id": media_id, "index": index, "articles": articles})

    def count(self) -> dict:
        """
        获取草稿总数
        开发者可以根据本接口来获取草稿的总数。此接口只统计数量，不返回草稿的具体内容。
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Draft_Box/Count_drafts.html

        :return: 返回的 JSON 数据包
        """
        return self._get("draft/count")

    def batchget(self) -> dict:
        """
        获取草稿列表
        新增草稿之后，开发者可以获取草稿的列表。
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Draft_Box/Get_draft_list.html

        :return: 返回的 JSON 数据包
        """
        return self._post("draft/batchget")



# -*- coding: utf-8 -*-

import time
import random
from datetime import datetime

from optionaldict import optionaldict

from wechatpy.utils import timezone
from wechatpy.pay.utils import get_external_ip, calculate_signature
from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatWithhold(BaseWeChatPayAPI):
    def apply_signing(
        self,
        plan_id,
        contract_code,
        contract_display_account,
        notify_url,
        version="1.0",
        clientip=None,
        deviceid=None,
        mobile=None,
        email=None,
        qq=None,
        request_serial=None,
        openid=None,
        creid=None,
        outerid=None,
    ):
        """
        申请签约 api

        https://pay.weixin.qq.com/wiki/doc/api/pap.php?chapter=18_1&index=1

        :param plan_id: 模板id 协议模板id，设置路径见开发步骤。
        :param contract_code: 签约协议号 商户侧的签约协议号，由商户生成
        :param contract_display_account: 用户账户展示名称 签约用户的名称，用于页面展示，页面样例可见案例与规范
        :param notify_url: 回调通知url 用于接收签约成功消息的回调通知地址，以http或https开头。
        :param version: 版本号 固定值1.0
        :param request_serial: 可选 请求序列号 商户请求签约时的序列号，商户侧须唯一。序列号主要用于排序，不作为查询条件
        :param clientip: 可选 客户端 IP 点分IP格式(客户端IP)
        :param deviceid: 可选 设备ID android填imei的一次md5; ios填idfa的一次md5
        :param mobile: 可选 手机号 用户手机号
        :param email: 可选 邮箱地址 用户邮箱地址
        :param qq: 可选 QQ号 用户QQ号
        :param openid: 可选 微信open ID 用户微信open ID
        :param creid: 可选 身份证号 用户身份证号
        :param outerid: 可选 商户侧用户标识 用户在商户侧的标识
        :return: 返回的结果数据字典
        """
        timestamp = int(time.time())
        if request_serial is None:
            request_serial = int(time.time() * 1000)
        data = {
            "appid": self.appid,
            "mch_id": self.mch_id,
            "sub_mch_id": self.sub_mch_id,
            "plan_id": plan_id,
            "contract_code": contract_code,
            "request_serial": request_serial,
            "contract_display_account": contract_display_account,
            "notify_url": notify_url,
            "version": version,
            "timestamp": timestamp,
            "clientip": clientip,
            "deviceid": deviceid,
            "mobile": mobile,
            "email": email,
            "qq": qq,
            "openid": openid,
            "creid": creid,
            "outerid": outerid,
        }
        data = optionaldict(data)
        sign = calculate_signature(data, self._client.api_key)
        data["sign"] = sign
        return {
            "base_url": f"{self._client.API_BASE_URL}papay/entrustweb",
            "data": data,
        }

    def query_signing(
        self,
        contract_id=None,
        plan_id=None,
        contract_code=None,
        openid=None,
        version="1.0",
    ):
        """
        查询签约关系 api

        :param contract_id: 可选 委托代扣协议id 委托代扣签约成功后由微信返回的委托代扣协议id，选择contract_id查询，则此参数必填
        :param plan_id: 可选 模板id 商户在微信商户平台配置的代扣模板id，选择plan_id+contract_code查询，则此参数必填
        :param contract_code: 可选 签约协议号 商户请求签约时传入的签约协议号，商户侧须唯一。选择plan_id+contract_code查询，则此参数必填
        :param openid: 可选 openid 用户标识，必须保证与传入appid对应
        :param version: 版本号 固定值1.0
        :return: 返回的结果信息
        """
        if not contract_id and not (plan_id and contract_code) and not (plan_id and openid):
            raise ValueError("contract_id and (plan_id, contract_code) and (plan_id, openid) must be a choice.")
        data = {
            "appid": self.appid,
            "mch_id": self.mch_id,
            "contract_id": contract_id,
            "plan_id": plan_id,
            "contract_code": contract_code,
            "openid": openid,
            "version": version,
            "nonce_str": None,
        }
        return self._post("papay/querycontract", data=data)

    def apply_deduct(
        self,
        body,
        total_fee,
        contract_id,
        notify_url,
        out_trade_no=None,
        detail=None,
        attach=None,
        fee_type="CNY",
        goods_tag=None,
        clientip=None,
        deviceid=None,
        mobile=None,
        email=None,
        qq=None,
        openid=None,
        creid=None,
        outerid=None,
    ):
        """
        申请扣款 api

        :param body: 商品描述 商品或支付单简要描述
        :param out_trade_no: 可选 商户订单号 商户系统内部的订单号,32个字符内、可包含字母, 其他说明见商户订单号
        :param total_fee: 总金额 订单总金额，单位为分，只能为整数，详见支付金额
        :param contract_id: 委托代扣协议id 签约成功后，微信返回的委托代扣协议id
        :param notify_url: 回调通知url 接受扣款结果异步回调通知的url
        :param detail: 可选 商品详情 商品名称明细列表
        :param attach: 可选 附加数据 附加数据，在查询API和支付通知中原样返回，该字段主要用于商户携带订单的自定义数据
        :param fee_type: 可选 货币类型 符合ISO 4217标准的三位字母代码，默认人民币：CNY
        :param goods_tag: 可选 商品标记 商品标记，代金券或立减优惠功能的参数，说明详见代金券或立减优惠
        :param clientip: 可选 客户端 IP 点分IP格式(客户端IP)
        :param deviceid: 可选 设备ID android填imei的一次md5; ios填idfa的一次md5
        :param mobile: 可选 手机号 用户手机号
        :param email: 可选 邮箱地址 用户邮箱地址
        :param qq: 可选 QQ号 用户QQ号
        :param openid: 可选 微信open ID 用户微信open ID
        :param creid: 可选 身份证号 用户身份证号
        :param outerid: 可选 商户侧用户标识 用户在商户侧的标识
        :return: 返回的结果信息
        """
        trade_type = "PAP"  # 交易类型 交易类型PAP-微信委托代扣支付
        timestamp = int(time.time())  # 10位时间戳
        spbill_create_ip = get_external_ip()  # 终端IP 调用微信支付API的机器IP
        if not out_trade_no:
            now = datetime.fromtimestamp(time.time(), tz=timezone("Asia/Shanghai"))
            out_trade_no = f"{self.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"

        data = {
            "appid": self.appid,
            "mch_id": self.mch_id,
            "body": body,
            "out_trade_no": out_trade_no,
            "total_fee": total_fee,
            "trade_type": trade_type,
            "contract_id": contract_id,
            "notify_url": notify_url,
            "detail": detail,
            "attach": attach,
            "fee_type": fee_type,
            "goods_tag": goods_tag,
            "clientip": clientip,
            "deviceid": deviceid,
            "mobile": mobile,
            "email": email,
            "qq": qq,
            "openid": openid,
            "creid": creid,
            "outerid": outerid,
            "timestamp": timestamp,
            "spbill_create_ip": spbill_create_ip,
        }
        return self._post("pay/pappayapply", data=data)

    def query_order(self, transaction_id=None, out_trade_no=None):
        """
        查询订单 api

        :param transaction_id: 二选一 微信订单号 微信的订单号，优先使用
        :param out_trade_no: 二选一 商户订单号 商户系统内部的订单号，当没提供transaction_id时需要传这个。
        :return: 返回的结果信息
        """
        if not transaction_id and not out_trade_no:
            raise ValueError("transaction_id and out_trade_no must be a choice.")
        data = {
            "appid": self.appid,
            "mch_id": self.mch_id,
            "transaction_id": transaction_id,
            "out_trade_no": out_trade_no,
        }
        return self._post("pay/paporderquery", data=data)

    def apply_cancel_signing(
        self,
        contract_id=None,
        plan_id=None,
        contract_code=None,
        contract_termination_remark=None,
        version="1.0",
    ):
        """
        申请解约

        https://pay.weixin.qq.com/wiki/doc/api/pap.php?chapter=18_4&index=6

        :param contract_id: 合同ID
        :param plan_id: 模板ID
        :param contract_code: 合同号
        :param contract_termination_remark: 解约原因
        :param version: 版本号
        :return:
        """
        if not (contract_id or (plan_id and contract_code)):
            raise ValueError("contract_id and (plan_id, contract_code) must be a choice.")
        data = {
            "appid": self.appid,
            "mch_id": self.mch_id,
            "plan_id": plan_id,
            "contract_code": contract_code,
            "contract_id": contract_id,
            "contract_termination_remark": contract_termination_remark,
            "version": version,
            "nonce_str": None,
        }
        return self._post("papay/deletecontract", data=data)



# -*- coding: utf-8 -*-


import time

from wechatpy.client.api.base import BaseWeChatAPI
from wechatpy.utils import WeChatSigner


class WeChatJSAPI(BaseWeChatAPI):
    """
    https://work.weixin.qq.com/api/doc#90001/90144/90539
    """

    def get_ticket(self):
        """
        获取企业的jsapi_ticket

        https://work.weixin.qq.com/api/doc#90001/90144/90539/获取企业的jsapi_ticket/

        :return: 返回的 JSON 数据包
        """
        return self._get("get_jsapi_ticket")

    def get_jsapi_signature(self, noncestr, ticket, timestamp, url):
        """
        获取 JSAPI 签名

        https://work.weixin.qq.com/api/doc#90001/90144/90539/签名算法/

        :param noncestr: nonce string
        :param ticket: JS-SDK ticket
        :param timestamp: 时间戳
        :param url: URL
        :return: 签名
        """
        data = [
            f"noncestr={noncestr}",
            f"jsapi_ticket={ticket}",
            f"timestamp={timestamp}",
            f"url={url}",
        ]
        signer = WeChatSigner(delimiter=b"&")
        signer.add_data(*data)
        return signer.signature

    def get_agent_ticket(self):
        """
        获取应用的jsapi_ticket

        https://work.weixin.qq.com/api/doc#90001/90144/90539/获取应用的jsapi_ticket/

        :return: 返回的 JSON 数据包
        """
        return self._get("ticket/get", params={"type": "agent_config"})

    def get_jsapi_ticket(self):
        """
        获取微信 JS-SDK ticket

        该方法会通过 session 对象自动缓存管理 ticket

        :return: ticket
        """
        ticket_key = f"{self._client.corp_id}_jsapi_ticket"
        expires_at_key = f"{self._client.corp_id}_jsapi_ticket_expires_at"
        ticket = self.session.get(ticket_key)
        expires_at = self.session.get(expires_at_key) or 0
        if not ticket or expires_at < int(time.time()):
            jsapi_ticket = self.get_ticket()
            ticket = jsapi_ticket["ticket"]
            expires_at = int(time.time()) + int(jsapi_ticket["expires_in"])
            self.session.set(ticket_key, ticket)
            self.session.set(expires_at_key, expires_at)
        return ticket

    def get_agent_jsapi_ticket(self):
        """
        获取应用的jsapi_ticket

        该方法会通过 session 对象自动缓存管理 ticket

        :return: ticket
        """
        ticket_key = f"{self._client.corp_id}_agent_jsapi_ticket"
        expires_at_key = f"{self._client.corp_id}_agent_jsapi_ticket_expires_at"
        ticket = self.session.get(ticket_key)
        expires_at = self.session.get(expires_at_key) or 0
        if not ticket or expires_at < int(time.time()):
            jsapi_ticket = self.get_agent_ticket()
            ticket = jsapi_ticket["ticket"]
            expires_at = int(time.time()) + int(jsapi_ticket["expires_in"])
            self.session.set(ticket_key, ticket)
            self.session.set(expires_at_key, expires_at)
        return ticket



# -*- coding: utf-8 -*-


from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatWxa(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/"

    def create_qrcode(self, path, width=430):
        """
        创建小程序二维码（接口C：适用于需要的码数量较少的业务场景）
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/qr-code/wxacode.createQRCode.html
        """
        return self._post("cgi-bin/wxaapp/createwxaqrcode", data={"path": path, "width": width})

    def get_wxa_code(
        self,
        path,
        width=430,
        auto_color=False,
        line_color=None,
        is_hyaline=False,
    ):
        """
        创建小程序码（接口A: 适用于需要的码数量较少的业务场景）
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/qr-code/wxacode.get.html
        """
        if line_color is None:
            line_color = {"r": "0", "g": "0", "b": "0"}
        return self._post(
            "wxa/getwxacode",
            data={
                "path": path,
                "width": width,
                "auto_color": auto_color,
                "line_color": line_color,
                "is_hyaline": is_hyaline,
            },
        )

    def get_wxa_code_unlimited(
        self,
        scene,
        width=430,
        auto_color=False,
        line_color=None,
        page=None,
        is_hyaline=False,
    ):
        """
        创建小程序码（接口B：适用于需要的码数量极多，或仅临时使用的业务场景）
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/qr-code/wxacode.getUnlimited.html
        """
        if line_color is None:
            line_color = {"r": "0", "g": "0", "b": "0"}
        return self._post(
            "wxa/getwxacodeunlimit",
            data=optionaldict(
                scene=scene,
                page=page,
                width=width,
                auto_color=auto_color,
                line_color=line_color,
                is_hyaline=is_hyaline,
            ),
        )

    def send_template_message(
        self,
        user_id,
        template_id,
        data,
        form_id,
        page=None,
        color=None,
        emphasis_keyword=None,
    ):
        """
        ⚠️已废弃
        发送模板消息
        详情请参考
        https://mp.weixin.qq.com/debug/wxadoc/dev/api/notice.html
        """
        tpl_data = optionaldict(
            touser=user_id,
            template_id=template_id,
            page=page,
            form_id=form_id,
            data=data,
            color=color,
            emphasis_keyword=emphasis_keyword,
        )
        return self._post("cgi-bin/message/wxopen/template/send", data=tpl_data)

    def send_subscribe_message(self, user_id, template_id, data, page=None):
        """
        发送小程序订阅消息
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/subscribe-message/subscribeMessage.send.html
        """
        subs_data = optionaldict(touser=user_id, template_id=template_id, page=page, data=data)
        return self._post("cgi-bin/message/subscribe/send", data=subs_data)

    def modify_domain(
        self,
        action,
        request_domain=(),
        wsrequest_domain=(),
        upload_domain=(),
        download_domain=(),
    ):
        """
        修改小程序服务器授权域名
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489138143_WPbOO

        :param action: 增删改查的操作类型，仅支持 'add', 'delete', 'set', 'get'
        :param request_domain: request 合法域名
        :param wsrequest_domain: socket 合法域名
        :param upload_domain: upload file 合法域名
        :param download_domain: download file 合法域名
        """
        return self._post(
            "wxa/modify_domain",
            data={
                "action": action,
                "requestdomain": request_domain,
                "wsrequestdomain": wsrequest_domain,
                "uploaddomain": upload_domain,
                "downloaddomain": download_domain,
            },
        )

    def bind_tester(self, wechat_id):
        """
        绑定微信用户成为小程序体验者
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140588_nVUgx

        :param wechat_id: 微信号
        """
        return self._post("wxa/bind_tester", data={"wechatid": wechat_id})

    def unbind_tester(self, wechat_id):
        """
        解除绑定小程序的体验者
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140588_nVUgx

        :param wechat_id: 微信号
        """
        return self._post("wxa/unbind_tester", data={"wechatid": wechat_id})

    def commit(self, template_id, ext_json, version, description):
        """
        为授权的小程序账号上传小程序代码
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4

        :param template_id: 代码库中的代码模板 ID
        :param ext_json: 第三方自定义的配置
        :param version: 代码版本号，开发者可自定义
        :param description: 代码描述，开发者可自定义
        """
        return self._post(
            "wxa/commit",
            data={"template_id": template_id, "ext_json": ext_json, "user_version": version, "user_desc": description},
        )

    def get_qrcode(self):
        """
        获取体验小程序的体验二维码
        返回 Response 类型，header 中带有 Content-Type 与 Content-disposition 类型
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4

        :rtype: requests.Response
        """
        return self._get("wxa/get_qrcode")

    def get_url_scheme(self, is_expire=False, expire_type=0, expire_time=None, expire_interval=None, jump_wxa=()):
        """
        获取小程序 scheme 码，适用于短信、邮件、外部网页、微信内等拉起小程序的业务场景。
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/url-scheme/urlscheme.generate.html

        :param is_expire, 到期失效：True，永久有效：False 默认False， 永久有效上限10万
        :param expire_type, 失效时间：0，失效间隔天数：1
        :param expire_time, 到期失效的 scheme 码的失效时间, Unix 时间戳, is_expire 为 true 且 expire_type 为 0 时必填
        :param expire_interval, 到期失效的 scheme 码的失效间隔天数。生成的到期失效 scheme 码在该间隔时间到达前有效。最长间隔天数为365天。
                                is_expire 为 true 且 expire_type 为 1 时必填。
        :param jump_wxa, 跳转到的目标小程序信息。

        :rtype: requests.Response
        """
        return self._post(
            "wxa/generatescheme",
            data=optionaldict(
                jump_wxa=jump_wxa,
                is_expire=is_expire,
                expire_type=expire_type,
                expire_time=expire_time,
                expire_interval=expire_interval,
            ),
        )

    def get_url_link(
        self,
        path=None,
        query=None,
        is_expire=False,
        expire_type=0,
        expire_time=None,
        expire_interval=None,
        cloud_base=(),
    ):
        """
        获取小程序 URL Link，适用于短信、邮件、网页、微信内等拉起小程序的业务场景。
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/url-link/urllink.generate.html

        :param path, 通过 URL Link 进入的小程序页面路径，必须是已经发布的小程序存在的页面，不可携带 query
        :param query, 通过 URL Link 进入小程序时的query，最大1024个字符，只支持数字，大小写英文以及部分特殊字符
        :param is_expire, 到期失效：True，永久有效：False 默认False， 永久有效上限10万
        :param expire_type, 失效时间：0，失效间隔天数：1
        :param expire_time, 到期失效的 scheme 码的失效时间, Unix 时间戳, is_expire 为 true 且 expire_type 为 0 时必填
        :param expire_interval, 到期失效的 scheme 码的失效间隔天数。生成的到期失效 scheme 码在该间隔时间到达前有效。
                                最长间隔天数为365天。is_expire 为 true 且 expire_type 为 1 时必填
        :param cloud_base, 云开发静态网站自定义 H5 配置参数，可配置中转的云开发 H5 页面。不填默认用官方 H5 页面。

        :rtype: requests.Response
        """
        return self._post(
            "wxa/generate_urllink",
            data=optionaldict(
                path=path,
                query=query,
                is_expire=is_expire,
                expire_type=expire_type,
                expire_time=expire_time,
                expire_interval=expire_interval,
                cloud_base=cloud_base,
            ),
        )

    def get_category(self):
        """
        获取授权小程序账号的可选类目
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4

        :rtype: list[dict]
        """
        return self._get("wxa/get_category", result_processor=lambda x: x["category_list"])

    def get_page(self):
        """
        获取小程序的第三方提交代码的页面配置
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4

        :rtype: list
        """
        return self._get("wxa/get_page", result_processor=lambda x: x["page_list"])

    def submit_audit(self, data):
        """
        将第三方提交的代码包提交审核
        详情请参考
        https://developers.weixin.qq.com/doc/oplatform/Third-party_Platforms/Mini_Programs/code/submit_audit.html
        """
        return self._post("wxa/submit_audit", data=data)

    def undo_code_audit(self):
        """
        调用本接口可以撤回当前的代码审核单
        注意： 单个帐号每天审核撤回次数最多不超过 1 次，一个月不超过 10 次。
        https://developers.weixin.qq.com/doc/oplatform/Third-party_Platforms/Mini_Programs/code/undocodeaudit.html
        """
        return self._get("wxa/undocodeaudit")

    def revert_code_release(self):
        """
        调用本接口可以将小程序的线上版本进行回退
        注意：
        1. 如果没有上一个线上版本，将无法回退
        2. 只能向上回退一个版本，即当前版本回退后，不能再调用版本回退接口
        https://developers.weixin.qq.com/doc/oplatform/Third-party_Platforms/Mini_Programs/code/revertcoderelease.html
        """
        return self._get("wxa/revertcoderelease")

    def get_audit_status(self, auditid):
        """
        查询某个指定版本的审核状态
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4

        :param auditid: 审核编号
        :type auditid: int
        :return: 一个包含 status, reason 的 dict。status 0为审核成功，1为审核失败，2为审核中。
        """
        return self._post("wxa/get_auditstatus", data={"auditid": auditid})

    def get_latest_audit_status(self):
        """
        查询最近一次提交的审核状态
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4

        :return: 一个包含 status, reason, auditid 的 dict。status 0为审核成功，1为审核失败，2为审核中。
        """
        return self._get("wxa/get_latest_auditstatus")

    def release(self):
        """
        发布已通过审核的小程序
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4
        """
        return self._post("wxa/release", data={})

    def change_visit_status(self, close=False):
        """
        修改小程序线上代码的可见状态
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1489140610_Uavc4

        :param close: close 为 True 时会关闭小程序线上代码的可见状态。
        :type close: bool
        """
        return self._post("wxa/change_visitstatus", data={"action": "close" if close else "open"})

    def list_library_templates(self, offset=0, count=20):
        """
        ⚠️已废弃
        获取小程序模板库里，所有模板的ID与标题
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1500465446_j4CgR

        :param offset: 用于分页，表示起始量，最小值为0
        :type offset: int
        :param count: 用于分页，表示拉取数量，最大值为20
        :type count: int
        :return: 带有 total_count 与 list 的数据
        :rtype: dict
        """
        return self._post("cgi-bin/wxopen/template/library/list", data={"offset": offset, "count": count})

    def get_library_template(self, template_short_id):
        """
        ⚠️已废弃
        获取小程序模板库里，某个模板的详细信息
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1500465446_j4CgR

        :param template_short_id: 模板标题ID
        :rtype: dict
        """
        return self._post("cgi-bin/wxopen/template/library/get", data={"id": template_short_id})

    def list_templates(self, offset=0, count=20):
        """
        ⚠️已废弃
        获取本账号内所有模板
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1500465446_j4CgR

        :param offset: 用于分页，表示起始量，最小值为0
        :type offset: int
        :param count: 用于分页，表示拉取数量，最大值为20
        :type count: int
        :return: 模板列表
        :rtype: list[dict]
        """
        return self._post(
            "cgi-bin/wxopen/template/list",
            data={"offset": offset, "count": count},
            result_processor=lambda x: x["list"],
        )

    def add_template(self, template_short_id, keyword_id_list):
        """
        ⚠️已废弃
        组合模板，并将其添加至账号下的模板列表里
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1500465446_j4CgR

        :param template_short_id: 模板标题ID
        :param keyword_id_list: 按照顺序排列的模板关键词列表，最多10个
        :type keyword_id_list: list[int]
        :return: 模板ID
        """
        return self._post(
            "cgi-bin/wxopen/template/add",
            data={"id": template_short_id, "keyword_id_list": keyword_id_list},
            result_processor=lambda x: x["template_id"],
        )

    def del_template(self, template_id):
        """
        ⚠️已废弃
        删除本账号内某个模板
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1500465446_j4CgR

        :param template_id: 模板ID
        """
        return self._post("cgi-bin/wxopen/template/del", data={"template_id": template_id})

    def create_open(self, appid):
        """
        创建开放平台账号，并绑定公众号/小程序
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1498704199_1bcax

        :param appid: 授权公众号或小程序的 appid
        :return: 开放平台的 appid
        """
        return self._post("cgi-bin/open/create", data={"appid": appid}, result_processor=lambda x: x["open_appid"])

    def get_open(self, appid):
        """
        获取公众号/小程序所绑定的开放平台账号
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1498704199_1bcax

        :param appid: 授权公众号或小程序的 appid
        :return: 开放平台的 appid
        """
        return self._post("cgi-bin/open/get", data={"appid": appid}, result_processor=lambda x: x["open_appid"])

    def bind_open(self, appid, open_appid):
        """
        将公众号/小程序绑定到开放平台帐号下
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1498704199_1bcax

        :param appid: 授权公众号或小程序的 appid
        :param open_appid: 开放平台帐号 appid
        """
        return self._post("cgi-bin/open/bind", data={"appid": appid, "open_appid": open_appid})

    def unbind_open(self, appid, open_appid):
        """
        将公众号/小程序绑定到开放平台帐号下
        详情请参考
        https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&id=open1498704199_1bcax

        :param appid: 授权公众号或小程序的 appid
        :param open_appid: 开放平台帐号 appid
        """
        return self._post("cgi-bin/open/unbind", data={"appid": appid, "open_appid": open_appid})

    def code_to_session(self, js_code):
        """
        登录凭证校验。通过 wx.login() 接口获得临时登录凭证 code 后传到开发者服务器调用此接口完成登录流程。更多使用方法详见 小程序登录
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api/code2Session.html
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/login/auth.code2Session.html

        :param js_code:
        :return:
        """
        return self._get(
            "sns/jscode2session",
            params={
                "appid": self.appid,
                "secret": self.secret,
                "js_code": js_code,
                "grant_type": "authorization_code",
            },
        )

    def get_phone_number(self, code):
        """
        code换取用户手机号。 每个code只能使用一次，code的有效期为5min
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/phonenumber/phonenumber.getPhoneNumber.html

        :param code
        :return:
        """
        return self._post(
            "wxa/business/getuserphonenumber",
            data={
                "code": code,
            },
        )

    def check_image_security(self, media):
        """
        校验一张图片是否含有违法违规内容。
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/sec-check/security.imgSecCheck.html

        :param media: 要检测的图片文件，格式支持PNG、JPEG、JPG、GIF，图片尺寸不超过 750px x 1334px
        :return:
        """
        return self._post("wxa/img_sec_check", files={"media": media})

    def check_media_async(self, media_url: str, media_type: int):
        """异步校验图片/音频是否含有违法违规内容。

        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/sec-check/security.mediaCheckAsync.html

        应用场景举例：
        1. 语音风险识别：社交类用户发表的语音内容检测；
        2. 图片智能鉴黄：涉及拍照的工具类应用(如美拍，识图类应用)用户拍照上传检测；电商类商品上架图片检测；媒体类用户文章里的图片检测等；
        3. 敏感人脸识别：用户头像；媒体类用户文章里的图片检测；社交类用户上传的图片检测等。 频率限制：单个 appId 调用上限为 2000 次/分钟，200,000 次/天；文件大小限制：单个文件大小不超过10M


        :param media_url: 要检测的多媒体url
        :param media_type: 1:音频;2:图片
        :return: 返回的 JSON 数据包
        """
        return self._post("wxa/media_check_async", data={"media_url": media_url, "media_type": media_type})

    def check_text_security(self, content):
        """
        检查一段文本是否含有违法违规内容。
        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/sec-check/security.msgSecCheck.html

        :param content: 要检测的文本内容，长度不超过 500KB
        :return:
        """
        return self._post("wxa/msg_sec_check", data={"content": content})

    def speed_up_audit(self, auditid):
        """
        加急审核申请
        有加急次数的第三方可以通过该接口，对已经提审的小程序进行加急操作，加急后的小程序预计2-12小时内审完。
        https://developers.weixin.qq.com/doc/oplatform/Third-party_Platforms/Mini_Programs/code/speedup_audit.html
        """
        return self._post("wxa/speedupaudit", data={"auditid": auditid})

    def query_quota(self):
        """
        查询服务商的当月提审限额（quota）和加急次数
        服务商可以调用该接口，查询当月平台分配的提审限额和剩余可提审次数，以及当月分配的审核加急次数和剩余加急次数。（所有旗下小程序共用该额度）
        https://developers.weixin.qq.com/doc/oplatform/Third-party_Platforms/Mini_Programs/code/query_quota.html
        """
        return self._get("wxa/queryquota")

    def get_paid_unionid(self, openid, transaction_id=None, mch_id=None, out_trade_no=None):
        """
        用户支付完成后，获取该用户的 UnionId，无需用户授权。

        详情请参考
        https://developers.weixin.qq.com/miniprogram/dev/api-backend/open-api/user-info/auth.getPaidUnionId.html

        :param openid: 支付用户唯一标识
        :param transaction_id: 微信支付订单号，可选
        :param mch_id: 微信支付分配的商户号，和商户订单号配合使用，可选
        :param out_trade_no: 微信支付商户订单号，和商户号配合使用
        :return: 用户唯一标识 unionid
        """
        return self._get(
            "wxa/getpaidunionid",
            params={
                "openid": openid,
                "transaction_id": transaction_id,
                "mch_id": mch_id,
                "out_trade_no": out_trade_no,
            },
            result_processor=lambda x: x["unionid"],
        )

    def get_merchant_category(self) -> dict:
        """
        拉取门店小程序类目

        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#5
        """
        return self._get("wxa/get_merchant_category")

    def apply_merchant(self, data: dict) -> dict:
        """
        创建门店小程序
        创建门店小程序提交后需要公众号管理员确认通过后才可进行审核。如果主管理员24小时超时未确认，才能再次提交。
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#6

        :param data: 门店数据，具体请参考文档
        """
        return self._post("wxa/applywxastore", data=data)

    def get_merchant_audit_info(self) -> dict:
        """
        查询门店小程序审核结果
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#7
        """
        return self._get("wxa/get_merchant_audit_info")

    def modify_merchant(self, data: dict) -> dict:
        """
        修改门店小程序信息
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#8

        :param data: 具体请参考文档
        :return: 参考文档
        """
        return self._post("wxa/modify_merchant", data=data)

    def get_district(self) -> dict:
        """
        从腾讯地图拉取省市区信息
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#9
        :return: 参考文档
        """
        return self._get("wxa/get_district")

    def search_map_poi(self, data: dict) -> dict:
        """
        在腾讯地图中搜索门店
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#10

        :param data: 具体请参考文档
        :return: 参考文档
        """
        return self._post("wxa/search_map_poi", data=data)

    def create_map_poi(self, data: dict) -> dict:
        """
        在腾讯地图中创建门店
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#11

        :param data: 具体请参考文档
        :return: 参考文档
        """
        return self._post("wxa/create_map_poi", data=data)

    def add_store(self, data: dict) -> dict:
        """
        添加门店
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#12

        :param data: 具体请参考文档
        :return: 参考文档
        """
        return self._post("wxa/add_store", data=data)

    def update_store(self, data: dict) -> dict:
        """
        更新门店信息
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#13

        :param data: 具体请参考文档
        :return: 参考文档
        """
        return self._post("wxa/update_store", data=data)

    def get_store_info(self, poi_id: str) -> dict:
        """
        获取单个门店信息
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#14

        :param poi_id: 为门店小程序添加门店，审核成功后返回的门店id
        :return: 门店信息
        """
        return self._post("wxa/get_store_info", data={"poi_id": poi_id})

    def get_store_list(self, offset: int = 0, limit: int = 10) -> dict:
        """
        获取门店信息列表
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#15

        :param offset: 获取门店列表的初始偏移位置，从0开始计数
        :param limit: 获取门店个数
        :return: 参考文档
        """
        return self._post("wxa/get_store_list", data={"offset": offset, "limit": limit})

    def del_store(self, poi_id: str) -> dict:
        """
        删除门店
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Shop_Miniprogram_Interface.html#16

        :param poi_id: 为门店小程序添加门店，审核成功后返回的门店id
        :return: 参考文档
        """
        return self._post("wxa/del_store", data={"poi_id": poi_id})



# -*- coding: utf-8 -*-


from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI
from wechatpy.utils import to_text


class WeChatBatch(BaseWeChatAPI):
    """
    https://work.weixin.qq.com/api/doc#90000/90135/90979

    异步批量接口用于大批量数据的处理，提交后接口即返回，企业微信会在后台继续执行任务。
    执行完成后，企业微信后台会通过任务事件通知企业获取结果。事件的内容是加密的，解密过程请参考 [消息的加解密处理][signure]，任务事件请参考异步任务完成事件推送。
    目前，仅为通讯录更新提供了异步批量接口
    """

    def sync_user(self, url, token, encoding_aes_key, media_id, to_invite=True):
        """
        增量更新成员

        https://work.weixin.qq.com/api/doc#90000/90135/90980

        :param url: 企业应用接收企业微信推送请求的访问协议和地址，支持http或https协议
        :param token: 用于生成签名
        :param encoding_aes_key: 用于消息体的加密，是AES密钥的Base64编码
        :param media_id: 上传的csv文件的media_id
        :param to_invite: 是否邀请新建的成员使用企业微信（将通过微信服务通知或短信或邮件下发邀请，每天自动下发一次，最多持续3个工作日），默认值为true。
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "batch/syncuser",
            data={
                "media_id": media_id,
                "to_invite": to_invite,
                "callback": {
                    "url": url,
                    "token": token,
                    "encodingaeskey": encoding_aes_key,
                },
            },
        )

    def replace_user(self, url, token, encoding_aes_key, media_id, to_invite=True):
        """
        全量覆盖成员

        https://work.weixin.qq.com/api/doc#90000/90135/90981

        :param url: 企业应用接收企业微信推送请求的访问协议和地址，支持http或https协议
        :param token: 用于生成签名
        :param encoding_aes_key: 用于消息体的加密，是AES密钥的Base64编码
        :param media_id: 上传的csv文件的media_id
        :param to_invite: 是否邀请新建的成员使用企业微信（将通过微信服务通知或短信或邮件下发邀请，每天自动下发一次，最多持续3个工作日），默认值为true。
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "batch/replaceuser",
            data={
                "media_id": media_id,
                "to_invite": to_invite,
                "callback": {
                    "url": url,
                    "token": token,
                    "encodingaeskey": encoding_aes_key,
                },
            },
        )

    def replace_party(self, url, token, encoding_aes_key, media_id):
        """
        全量覆盖部门

        https://work.weixin.qq.com/api/doc#90000/90135/90982

        :param url: 企业应用接收企业微信推送请求的访问协议和地址，支持http或https协议
        :param token: 用于生成签名
        :param encoding_aes_key: 用于消息体的加密，是AES密钥的Base64编码
        :param media_id: 上传的csv文件的media_id
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "batch/replaceparty",
            data={
                "media_id": media_id,
                "callback": {
                    "url": url,
                    "token": token,
                    "encodingaeskey": encoding_aes_key,
                },
            },
        )

    def get_result(self, job_id):
        """
        获取异步任务结果

        https://work.weixin.qq.com/api/doc#90000/90135/90983

        :param job_id: 异步任务id，最大长度为64字符
        :return: 返回的 JSON 数据包
        """
        return self._get("batch/getresult", params={"jobid": job_id})

    def invite(self, user=None, party=None, tag=None):
        """
        邀请成员

        https://work.weixin.qq.com/api/doc#90000/90135/90975

        企业可通过接口批量邀请成员使用企业微信，邀请后将通过短信或邮件下发通知。

        :param user: 成员ID列表, 最多支持1000个。
        :param party: 成员ID列表, 最多支持100个。
        :param tag: 成员ID列表, 最多支持100个。
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(user=user, party=party, tag=tag)
        return self._post("batch/invite", data=data)



# -*- coding: utf-8 -*-


import base64
import urllib

from wechatpy.client.api.base import BaseWeChatAPI
from wechatpy.utils import to_binary, to_text


class WeChatDevice(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/device/"

    def send_message(self, device_type, device_id, user_id, content):
        """
        主动发送消息给设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-3

        :param device_type: 设备类型，目前为“公众账号原始ID”
        :param device_id: 设备ID
        :param user_id: 微信用户账号的openid
        :param content: 消息内容，BASE64编码
        :return: 返回的 JSON 数据包
        """
        content = to_text(base64.b64encode(to_binary(content)))
        return self._post(
            "transmsg",
            data={
                "device_type": device_type,
                "device_id": device_id,
                "open_id": user_id,
                "content": content,
            },
        )

    def send_status_message(self, device_type, device_id, user_id, msg_type, device_status):
        """
        第三方主动发送设备状态消息给微信终端
        详情请参考
        https://iot.weixin.qq.com/wiki/document-2_10.html

        :param device_type: 设备类型，目前为“公众账号原始ID”
        :param device_id: 设备ID
        :param user_id: 微信用户账号的openid
        :param msg_type: 消息类型：2--设备状态消息
        :param status: 设备状态：0--未连接， 1--已连接
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "transmsg",
            data={
                "device_type": device_type,
                "device_id": device_id,
                "open_id": user_id,
                "msg_type": msg_type,
                "device_status": device_status,
            },
        )

    def create_qrcode(self, device_ids):
        """
        获取设备二维码
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-4

        :param device_ids: 设备id的列表
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "create_qrcode",
            data={"device_num": len(device_ids), "device_id_list": device_ids},
        )

    def get_qrcode_url(self, ticket, data=None):
        """
        通过 ticket 换取二维码地址
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-4

        :param ticket: 二维码 ticket
        :param data: 额外数据
        :return: 二维码地址
        """
        url = f"https://we.qq.com/d/{ticket}"
        if data:
            if isinstance(data, (dict, tuple, list)):
                data = urllib.parse.urlencode(data)
            data = to_text(base64.b64encode(to_binary(data)))
            url = f"{url}#{data}"
        return url

    def bind(self, ticket, device_id, user_id):
        """
        绑定设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param ticket: 绑定操作合法性的凭证（由微信后台生成，第三方H5通过客户端jsapi获得）
        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post("bind", data={"ticket": ticket, "device_id": device_id, "openid": user_id})

    def unbind(self, ticket, device_id, user_id):
        """
        解绑设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param ticket: 绑定操作合法性的凭证（由微信后台生成，第三方H5通过客户端jsapi获得）
        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post("unbind", data={"ticket": ticket, "device_id": device_id, "openid": user_id})

    def compel_bind(self, device_id, user_id):
        """
        强制绑定用户和设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post("compel_bind", data={"device_id": device_id, "openid": user_id})

    force_bind = compel_bind

    def compel_unbind(self, device_id, user_id):
        """
        强制解绑用户和设备
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-7

        :param device_id: 设备id
        :param user_id: 用户对应的openid
        :return: 返回的 JSON 数据包
        """
        return self._post("compel_unbind", data={"device_id": device_id, "openid": user_id})

    force_unbind = compel_unbind

    def get_stat(self, device_id):
        """
        设备状态查询
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-8

        :param device_id: 设备id
        :return: 返回的 JSON 数据包
        """
        return self._get("get_stat", params={"device_id": device_id})

    def verify_qrcode(self, ticket):
        """
        验证二维码
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-9

        :param ticket: 设备二维码的ticket
        :return: 返回的 JSON 数据包
        """
        return self._post("verify_qrcode", data={"ticket": ticket})

    def get_user_id(self, device_type, device_id):
        """
        获取设备绑定openID
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-11

        :param device_type: 设备类型，目前为“公众账号原始ID”
        :param device_id: 设备id
        :return: 返回的 JSON 数据包
        """
        return self._get("get_openid", params={"device_type": device_type, "device_id": device_id})

    get_open_id = get_user_id

    def get_binded_devices(self, user_id):
        """
        通过openid获取用户在当前devicetype下绑定的deviceid列表
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-12

        :param user_id: 要查询的用户的openid
        :return: 返回的 JSON 数据包
        """
        return self._get("get_bind_device", params={"openid": user_id})

    get_bind_device = get_binded_devices

    def get_qrcode(self, product_id=1):
        """
        获取deviceid和二维码
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-4

        :param product_id: 设备的产品编号
        :return: 返回的 JSON 数据包
        """
        if product_id == "1" or product_id == 1:
            params = None
        else:
            params = {"product_id": product_id}

        return self._get("getqrcode", params=params)

    def authorize(self, devices, op_type=1):
        """
        设备授权
        详情请参考
        https://iot.weixin.qq.com/wiki/new/index.html?page=3-4-5

        :param devices: 设备信息的列表
        :param op_type: 请求操作的类型，限定取值为：0：设备授权 1：设备更新
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "authorize_device",
            data={
                "device_num": len(devices),
                "device_list": devices,
                "op_type": op_type,
            },
        )



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI


class MerchantShelf(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def add(self, name, banner, shelf_data):
        return self._post(
            "merchant/shelf/add",
            data={"shelf_name": name, "shelf_banner": banner, "shelf_data": shelf_data},
        )

    def delete(self, shelf_id):
        return self._post("merchant/shelf/del", data={"shelf_id": shelf_id})

    def update(self, shelf_id, name, banner, shelf_data):
        return self._post(
            "merchant/shelf/add",
            data={
                "shelf_id": shelf_id,
                "shelf_name": name,
                "shelf_banner": banner,
                "shelf_data": shelf_data,
            },
        )

    def get_all(self):
        res = self._get("merchant/shelf/getall", result_processor=lambda x: x["shelves"])
        return res

    def get(self, shelf_id):
        return self._post("merchant/shelf/getbyid", data={"shelf_id": shelf_id})



from wechatpy.crypto import BasePrpCrypto, BaseWeChatCrypto
from wechatpy.work.exceptions import InvalidCorpIdException


class PrpCrypto(BasePrpCrypto):
    def encrypt(self, text, corp_id):
        return self._encrypt(text, corp_id)

    def decrypt(self, text, corp_id):
        return self._decrypt(text, corp_id, InvalidCorpIdException)


class WeChatCrypto(BaseWeChatCrypto):
    def __init__(self, token, encoding_aes_key, corp_id):
        super(WeChatCrypto, self).__init__(token, encoding_aes_key, corp_id)
        self.corp_id = corp_id

    def check_signature(self, signature, timestamp, nonce, echo_str):
        return self._check_signature(signature, timestamp, nonce, echo_str, PrpCrypto)

    def encrypt_message(self, msg, nonce, timestamp=None):
        return self._encrypt_message(msg, nonce, timestamp, PrpCrypto)

    def decrypt_message(self, msg, signature, timestamp, nonce):
        return self._decrypt_message(msg, signature, timestamp, nonce, PrpCrypto)



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatPoi(BaseWeChatAPI):
    """微信门店

    https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Store_Interface.html
    """

    def add(self, poi_data):
        """
        创建门店

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Store_Interface.html#7

        :param poi_data: 门店信息字典
        :return: 返回的 JSON 数据包
        """
        return self._post("poi/addpoi", data=poi_data)

    def get(self, poi_id):
        """
        查询门店信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Store_Interface.html#9

        :param poi_id: 门店 ID
        :return: 返回的 JSON 数据包
        """
        return self._post("poi/getpoi", data={"poi_id": poi_id})

    def list(self, begin=0, limit=20):
        """
        查询门店列表

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Store_Interface.html#10

        :param begin: 开始位置，0 即为从第一条开始查询
        :param limit: 返回数据条数，最大允许50，默认为20
        :return: 返回的 JSON 数据包
        """
        return self._post("poi/getpoilist", data={"begin": begin, "limit": limit})

    def update(self, poi_data):
        """
        修改门店服务信息

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Store_Interface.html#11

        :param poi_data: 门店信息字典
        :return: 返回的 JSON 数据包
        """
        return self._post("poi/updatepoi", data=poi_data)

    def delete(self, poi_id):
        """
        删除门店

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Store_Interface.html#12

        :param poi_id: 门店 ID
        :return: 返回的 JSON 数据包
        """
        return self._post("poi/delpoi", data={"poi_id": poi_id})

    def get_categories(self):
        """
        获取微信门店类目表

        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/WeChat_Stores/WeChat_Store_Interface.html#13

        :return: 门店类目表
        """
        res = self._get("api_getwxcategory", result_processor=lambda x: x["category_list"])
        return res



# -*- coding: utf-8 -*-


from wechatpy.work.client import WeChatClient  # NOQA
from wechatpy.work.crypto import WeChatCrypto  # NOQA
from wechatpy.work.parser import parse_message  # NOQA
from wechatpy.work.replies import create_reply  # NOQA



# -*- coding: utf-8 -*-


import requests

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMedia(BaseWeChatAPI):
    """
    素材管理

    https://work.weixin.qq.com/api/doc#90000/90135/91054
    """

    def upload(self, media_type, media_file):
        """
        上传临时素材

        https://work.weixin.qq.com/api/doc#90000/90135/90253

        :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和普通文件（file）
        :param media_file: 要上传的文件，一个 File-object
        :return: 返回的 JSON 数据包
        """
        return self._post("media/upload", params={"type": media_type}, files={"media": media_file})

    def upload_img(self, image_file):
        """
        上传永久图片

        https://work.weixin.qq.com/api/doc#90000/90135/90256

        上传图片得到图片URL，该URL永久有效。
        返回的图片URL，仅能用于图文消息（mpnews）正文中的图片展示；若用于非企业微信域名下的页面，图片将被屏蔽。
        每个企业每天最多可上传100张图片。
        图片文件大小应在 5B ~ 2MB 之间。

        :return: 返回的 JSON 数据包
        """
        return self._post("media/uploadimg", files={"media": image_file})

    def get_url(self, media_id):
        """
        获取临时素材

        https://work.weixin.qq.com/api/doc#90000/90135/90254

        :param media_id: 媒体文件id
        :return: 临时素材下载地址
        """
        parts = (
            "https://qyapi.weixin.qq.com/cgi-bin/media/get",
            "?access_token=",
            self.access_token,
            "&media_id=",
            media_id,
        )
        return "".join(parts)

    def get_jssdk_url(self, media_id):
        """
        获取高清语音素材

        https://work.weixin.qq.com/api/doc#90000/90135/90255

        :param media_id: 通过JSSDK的uploadVoice接口上传的语音文件id
        :return: 高清语音素材下载地址
        """
        parts = (
            "https://qyapi.weixin.qq.com/cgi-bin/media/get/jssdk",
            "?access_token=",
            self.access_token,
            "&media_id=",
            media_id,
        )
        return "".join(parts)

    def download(self, media_id):
        """
        获取临时素材文件

        https://work.weixin.qq.com/api/doc#90000/90135/90254

        :param media_id: 媒体文件id
        :return: requests 的 Response 实例
        """
        return requests.get(self.get_url(media_id))



#!/usr/bin/env python3
# -*- coding: utf8 -*-

# Date: 2020/05/06
# Author: huimingz
# Contact: huimingz12@outlook.com

"""
企业微信服务端电子发票API

企业微信文档：https://work.weixin.qq.com/api/doc/90000/90135/90283
"""

from typing import Dict, List

from wechatpy.client.api.base import BaseWeChatAPI

# 发票初始状态，未锁定
INVOICE_REIMBURSE_INIT = "INVOICE_REIMBURSE_INIT"
# 发票已锁定，无法重复提交报销
INVOICE_REIMBURSE_LOCK = "INVOICE_REIMBURSE_LOCK"
# 发票已核销，从用户卡包中移除
INVOICE_REIMBURSE_CLOSURE = "INVOICE_REIMBURSE_CLOSURE"

invoice_reimburse_status_set = {INVOICE_REIMBURSE_INIT, INVOICE_REIMBURSE_LOCK, INVOICE_REIMBURSE_CLOSURE}


class WeChatInvoice(BaseWeChatAPI):
    """企业微信电子发票API"""

    def get_info(self, card_id: str, encrypt_code: str) -> dict:
        """查询电子发票

        参考：https://work.weixin.qq.com/api/doc/90000/90135/90284

        接口说明：报销方在获得用户选择的电子发票标识参数后，可以通过该接口查询电子发票的
        结构化信息，并获取发票PDF文件。

        **权限说明**：仅认证的企业微信账号有接口权限

        返回结果参数说明请查看官方文档。

        :param card_id: 发票id
        :param encrypt_code: 加密code
        :return: 发票信息
        """
        url = "card/invoice/reimburse/getinvoiceinfo"
        data = {"card_id": card_id, "encrypt_code": encrypt_code}
        return self._post(url, data=data)

    def get_info_batch(self, item_list: List[Dict[str, str]]) -> dict:
        """批量查询电子发票

        参考：https://work.weixin.qq.com/api/doc/90000/90135/90287

        报销方在获得用户选择的电子发票标识参数后，可以通过该接口批量查询电子发票的结构化信息。

        **权限说明**：
        仅认证的企业微信账号并且企业激活人数超过200的企业才有接口权限，如果认证的企业
        激活人数不超过200人请联系企业微信客服咨询。

        返回结果参数说明请查看官方文档。

        :param item_list: 发票列表，示例：
            [{'card_id': 'id', 'encrypt_code': 'code'}...]
        :return: 电子发票信息
        """
        if not item_list:
            raise ValueError("the item_list cannot be empty")

        url = "card/invoice/reimburse/getinvoiceinfobatch"
        data = {"item_list": item_list}
        return self._post(url, data=data)

    def update_status(self, card_id: str, encrypt_code: str, reimburse_status: str) -> dict:
        """更新发票状态

        参考：https://work.weixin.qq.com/api/doc/90000/90135/90285

        接口说明：报销企业和报销服务商可以通过该接口对某一张发票进行锁定、解锁和报销操作。

        **权限说明**：仅认证的企业微信账号有接口权限

        **发报销状态如下：**

        * **锁定**：电子发票进入了企业的报销流程时应该执行锁定操作，执行锁定操作后的电子
          发票仍然会存在于用户卡包内，但无法重复提交报销。
        * **解锁**：当电子发票由于各种原因，无法完成报销流程时，应执行解锁操作。执行锁定
          操作后的电子发票将恢复可以被提交的状态。
        * **报销**：当电子发票报销完成后，应该使用本接口执行报销操作。执行报销操作后的
          电子发票将从用户的卡包中移除，用户可以在卡包的消息中查看到电子发票的核销信息。
          注意，报销为不可逆操作，请开发者慎重调用。

        参数 ``reimburse_status`` 选项如下：

        * ``INVOICE_REIMBURSE_INIT`` 表示发票初始状态，未锁定；
        * ``INVOICE_REIMBURSE_LOCK`` 表示发票已锁定，无法重复提交报销;
        * ``INVOICE_REIMBURSE_CLOSURE`` 表示发票已核销，从用户卡包中移除。

        :param card_id: 发票id
        :param encrypt_code: 加密code
        :param reimburse_status: 发报销状态
        :return: 更新结果
        """
        self._validate_invoice_reimburse(reimburse_status)
        url = "card/invoice/reimburse/updateinvoicestatus"
        data = {"card_id": card_id, "encrypt_code": encrypt_code, "reimburse_status": reimburse_status}
        return self._post(url, data=data)

    def update_status_batch(self, openid: str, reimburse_status: str, invoice_list: List[Dict[str, str]]) -> dict:
        """批量更新发票状态

        参考：https://work.weixin.qq.com/api/doc/90000/90135/90286

        **接口说明**：发票平台可以通过该接口对某个成员的一批发票进行锁定、解锁和报销操作。
        注意，报销状态为不可逆状态，请开发者慎重调用。

        **权限说明**：仅认证的企业微信账号有接口权限

        **注意**：

        * 报销方须保证在报销、锁定、解锁后及时将状态同步至微信侧，保证用户发票可以正常使用
        * 批量更新发票状态接口为事务性操作，如果其中一张发票更新失败，列表中的其它发票状态
          更新也会无法执行，恢复到接口调用前的状态

        ------------------------

        参数 ``reimburse_status`` 选项如下：

        * ``INVOICE_REIMBURSE_INIT`` 表示发票初始状态，未锁定；
        * ``INVOICE_REIMBURSE_LOCK`` 表示发票已锁定，无法重复提交报销;
        * ``INVOICE_REIMBURSE_CLOSURE`` 表示发票已核销，从用户卡包中移除。

        :param openid: 用户openid，可用“userid与openid互换接口”获取
        :param reimburse_status: 发票报销状态
        :param invoice_list: 发票列表，必须全部属于同一个openid，示例：
            [{'card_id': 'id', 'encrypt_code': 'code'}...]
        :return: 更新结果
        """
        self._validate_invoice_reimburse(reimburse_status)
        if not invoice_list:
            raise ValueError("the invoice list cannot be empty")

        url = "card/invoice/reimburse/updatestatusbatch"
        data = {"openid": openid, "reimburse_status": reimburse_status, "invoice_list": invoice_list}
        return self._post(url, data=data)

    @staticmethod
    def _validate_invoice_reimburse(status: str) -> None:
        if not status:
            raise ValueError("the invoice reimburse_status cannot be empty")
        if status not in invoice_reimburse_status_set:
            raise ValueError(f"the invoice defer_status must be in the set of {invoice_reimburse_status_set!r}")



# -*- coding: utf-8 -*-


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMenu(BaseWeChatAPI):
    """
    自定义菜单

    https://work.weixin.qq.com/api/doc#90000/90135/90230
    """

    def create(self, agent_id, menu_data):
        """
        创建菜单

        https://work.weixin.qq.com/api/doc#90000/90135/90231

        :param agent_id: 应用id
        """
        return self._post("menu/create", params={"agentid": agent_id}, data=menu_data)

    def get(self, agent_id):
        """
        获取菜单

        https://work.weixin.qq.com/api/doc#90000/90135/90232

        :param agent_id: 应用id
        """
        return self._get("menu/get", params={"agentid": agent_id})

    def delete(self, agent_id):
        """
        删除菜单

        https://work.weixin.qq.com/api/doc#90000/90135/90233

        :param agent_id: 应用id
        """
        return self._get("menu/delete", params={"agentid": agent_id})

    def update(self, agent_id, menu_data):
        self.delete(agent_id)
        return self.create(agent_id, menu_data)



import time
import json
import dataclasses
from typing import Dict, Optional
from dataclasses import dataclass

from wechatpy.utils import random_string


@dataclass
class Scheme:
    def to_dict(self) -> Dict:
        return dataclasses.asdict(self)


@dataclass
class JsApiCardExt(Scheme):
    """本类用于指示 jsapi 中批量添加卡券接口的某个参数格式.

    参数含义: https://developers.weixin.qq.com/doc/offiaccount/OA_Web_Apps/JS-SDK.html#65

    文档中要求本结构应为 JSON 字符串, 因此使用时注意转换:

    .. highlight:: python
    .. code-block:: python

        from wechatpy import WeChatClient
        from wechatpy.schemes import JsApiCardExt

        client = WeChatClient('appid', 'appsecret')
        card_ext = client.jsapi.get_jsapi_add_card_params(
            card_ticket=card_ticket, timestamp=timestamp, card_id=card_id, nonce_str=nonce_str
        )
        card_ext_json = card_ext.to_json()
    """

    signature: str
    code: str = ""
    openid: str = ""
    timestamp: str = ""
    nonce_str: str = ""
    fixed_begintimestamp: Optional[int] = None
    outer_str: str = ""

    def __post_init__(self):
        self.timestamp = self.timestamp or str(int(time.time()))
        self.nonce_str = self.nonce_str or random_string()

    def to_json(self) -> str:
        d = {k: v for k, v in dataclasses.asdict(self).items() if v}
        return json.dumps(d, ensure_ascii=False)



# -*- coding: utf-8 -*-


from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMessage(BaseWeChatAPI):
    """
    发送应用消息

    https://work.weixin.qq.com/api/doc#90000/90135/90236

    支持：
    * 文本消息
    * 图片消息
    * 语音消息
    * 视频消息
    * 文件消息
    * 文本卡片消息
    * 图文消息
    * 图文消息（mpnews）
    * markdown消息
    * 小程序通知消息
    """

    def send(self, agent_id, user_ids, party_ids="", tag_ids="", msg=None):
        """
        通用的消息发送接口。msg 内需要指定 msgtype 和对应类型消息必须的字段。
        如果部分接收人无权限或不存在，发送仍然执行，但会返回无效的部分（即invaliduser或invalidparty或invalidtag），常见的原因是接收人不在应用的可见范围内。
        user_ids、party_ids、tag_ids 不能同时为空，后面不再强调。

        :param agent_id: 必填，企业应用的id，整型。可在应用的设置页面查看。
        :param user_ids: 成员ID列表。
        :param party_ids: 部门ID列表。
        :param tag_ids: 标签ID列表。
        :param msg: 发送消息的 dict 对象
        :type msg: dict | None
        :return: 接口调用结果
        """
        msg = msg or {}
        if isinstance(user_ids, (tuple, list)):
            user_ids = "|".join(user_ids)
        if isinstance(party_ids, (tuple, list)):
            party_ids = "|".join(party_ids)
        if isinstance(tag_ids, (tuple, list)):
            tag_ids = "|".join(tag_ids)

        data = {
            "touser": user_ids,
            "toparty": party_ids,
            "totag": tag_ids,
            "agentid": agent_id,
        }
        data.update(msg)
        return self._post("message/send", data=data)

    def send_text(self, agent_id, user_ids, content, party_ids="", tag_ids="", safe=0):
        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={"msgtype": "text", "text": {"content": content}, "safe": safe},
        )

    def send_text_card(
        self,
        agent_id,
        user_ids,
        title,
        description,
        url,
        btntxt="详情",
        party_ids="",
        tag_ids="",
    ):
        """文本卡片消息

        https://work.weixin.qq.com/api/doc#90000/90135/90236/文本卡片消息

        请求示例：
        {
           "touser" : "UserID1|UserID2|UserID3",
           "toparty" : "PartyID1 | PartyID2",
           "totag" : "TagID1 | TagID2",
           "msgtype" : "textcard",
           "agentid" : 1,
           "textcard" : {
                "title" : "领奖通知",
                "description" : "<div class=\"gray\">2016年9月26日</div> <div class=\"normal\">恭喜你抽中iPhone 7一台，
                    领奖码：xxxx</div><div class=\"highlight\">请于2016年10月10日前联系行政同事领取</div>",
                "url" : "URL",
                "btntxt":"更多"
           }
        }

        特殊说明：
        卡片消息的展现形式非常灵活，支持使用br标签或者空格来进行换行处理，也支持使用div标签来使用不同的字体颜色，
        目前内置了3种文字颜色：灰色(gray)、高亮(highlight)、默认黑色(normal)，将其作为div标签的class属性即可，
        具体用法请参考上面的示例。

        :param agent_id: 必填，企业应用的id，整型。可在应用的设置页面查看。
        :param user_ids: 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。
        :param title: 标题，不超过128个字节，超过会自动截断。
        :param description: 必填，描述，不超过512个字节，超过会自动截断
        :param url: 必填，点击后跳转的链接。
        :param btntxt: 按钮文字。 默认为“详情”， 不超过4个文字，超过自动截断。
        :param party_ids: 部门ID列表。
        :param tag_ids: 标签ID列表。
        """
        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={
                "msgtype": "textcard",
                "textcard": {
                    "title": title,
                    "description": description,
                    "url": url,
                    "btntxt": btntxt,
                },
            },
        )

    def send_image(self, agent_id, user_ids, media_id, party_ids="", tag_ids="", safe=0):
        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={"msgtype": "image", "image": {"media_id": media_id}, "safe": safe},
        )

    def send_voice(self, agent_id, user_ids, media_id, party_ids="", tag_ids="", safe=0):
        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={"msgtype": "voice", "voice": {"media_id": media_id}, "safe": safe},
        )

    def send_video(
        self,
        agent_id,
        user_ids,
        media_id,
        title=None,
        description=None,
        party_ids="",
        tag_ids="",
        safe=0,
    ):
        video_data = optionaldict()
        video_data["media_id"] = media_id
        video_data["title"] = title
        video_data["description"] = description

        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={"msgtype": "video", "video": dict(video_data), "safe": safe},
        )

    def send_file(self, agent_id, user_ids, media_id, party_ids="", tag_ids="", safe=0):
        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={"msgtype": "file", "file": {"media_id": media_id}, "safe": safe},
        )

    def send_articles(self, agent_id, user_ids, articles, party_ids="", tag_ids=""):
        articles_data = []
        for article in articles:
            articles_data.append(
                {
                    "title": article["title"],
                    "description": article["description"],
                    "url": article["url"],
                    "picurl": article["image"],
                }
            )
        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={"msgtype": "news", "news": {"articles": articles_data}},
        )

    def send_mp_articles(self, agent_id, user_ids, articles, party_ids="", tag_ids="", safe=0):
        articles_data = []
        for article in articles:
            articles_data.append(
                {
                    "thumb_media_id": article["thumb_media_id"],
                    "author": article["author"],
                    "title": article["title"],
                    "content": article["content"],
                    "content_source_url": article["content_source_url"],
                    "digest": article["digest"],
                    "show_cover_pic": article["show_cover_pic"],
                }
            )
        return self.send(
            agent_id,
            user_ids,
            party_ids,
            tag_ids,
            msg={
                "msgtype": "mpnews",
                "mpnews": {"articles": articles_data},
                "safe": safe,
            },
        )

    def send_markdown(self, agent_id, user_ids, content, party_ids="", tag_ids=""):
        """markdown消息

        https://work.weixin.qq.com/api/doc#90000/90135/90236/markdown%E6%B6%88%E6%81%AF

        > 目前仅支持markdown语法的子集
        > 微工作台（原企业号）不支持展示markdown消息

        :param agent_id: 企业应用的id，整型。可在应用的设置页面查看
        :type agent_id: string
        :param content: markdown内容，最长不超过2048个字节，必须是utf8编码
        :type content: string
        :param user_ids: 成员ID列表（消息接收者，多个接收者用‘|’分隔，最多支持1000个）。
            特殊情况：指定为@all，则向关注该企业应用的全部成员发送
        :type user_ids: list or tuple or string
        :param party_ids: 部门ID列表，最多支持100个。当touser为@all时忽略本参数
        :type party_ids: list or tuple or string
        :param tag_ids: 标签ID列表，最多支持100个。当touser为@all时忽略本参数
        :type tag_ids: list or tuple or string
        :return: 接口调用结果
        :rtype: dict
        """
        msg = {"msgtype": "markdown", "markdown": {"content": content}}
        return self.send(agent_id, user_ids, party_ids, tag_ids, msg=msg)

    def update_taskcard(self, agent_id, user_ids, task_id, replace_name):
        """更新任务卡片消息状态

        https://work.weixin.qq.com/api/doc/90000/90135/91579

        Args:
            agent_id (integer): 应用的agentid
            user_ids (list): 企业的成员ID列表（消息接收者，最多支持1000个）。
            task_id (string): 发送任务卡片消息时指定的task_id
            replace_name (string): 设置替换文案

        Returns:
            dict: 接口返回结果
        """
        data = {"userids": user_ids, "agentid": agent_id, "task_id": task_id, "replace_name": replace_name}
        return self._post("message/update_taskcard", data=data)
